/*
*
*  Copyright (c) 2005 Michael Stock. All Rights Reserved.
*
*  This file is part of Stirf.
*
*  Stirf is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  Stirf is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this software; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/*
	11.09.2006 modifications by Happy (Till Hepke)
	-removed startup message
	-epg fix and auto rename can now be used independently
	-fixed manual mode for German firmware (new folders ar now called "GRUPPE 01")
	-fixed manual mode where less than the maximum number of entries (30) were shown, increased max to 100
	-GMT offset now chosen automatically, using source from Firebird's GMT Bug workaround
	-removed hotkey for menu (EXIT key), now menu only with TSR/TAP commander
	
	-trap illegal characters now also deletes binary characters from filenames while renaming
	-GROUP 01 folder can now be configured
	-file renaming with double titles are avoided
	-5200c headers supported
	
	01.10.2006 
	-fixed GMT-Offset of endtime written to the file header
	
	30.01.2007
	-removed TAP_EnterNormal in TAP_Main to fix the deleted OSD upon scheduled startup of the TAP
*/

#define ID_STIRF 0x86574285
#define STIRFBOXITEMSPERPAGE 3
#define TAP_NAME "Euro-Stirf v1.32 (mod by Happy)"
#define ABS(x)	(((x)>0)?(x):(-(x)))
// topfield seems to use 257 bytes of title/subtitle data as opposed to the 129
// they claim. hence the spare room to search for data in.
// we need two more bytes to get the show's title length, so we read 259 from the rec file
#define READ_BUFFER_SIZE 259
#define SUBT_AREA_SIZE 257
#define EXT_INFO_AREA_SIZE 1024
#define NUM_EVENT_DATA 16 // maximum number of subtitles or extended info's that are looked for in files
#define MAX_NUM_FILES 100
#define INI_MAX_SIZE 1024
#define SCROLL_START_DELAY 300 // Time (in units of 10ms) between pressing a button and scrolling starting
#define SCROLL_LETTER_DELAY 13 // Time (in units of 10ms) between letters during scrolling
//#define GROUP01 "GRUPPE 01"     // name of folder in German changed by Happy
//#define GROUP01 "GROUP 01"     // name of folder in German changed by Happy
static char GROUP01[20];

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "tap.h"
#include "key.h"
#include "DirListToolsForStirf.c"
#include "stirf.gd"
#include "stirfonborder.gd"
#include "stirfoffborder.gd"
#include "finnishhelp.gd"
#include "flash.h"

TAP_ID( ID_STIRF );
TAP_PROGRAM_NAME(TAP_NAME);
TAP_AUTHOR_NAME("mike_perth/Happy");
TAP_DESCRIPTION("Puts EPG SubTitles into Rec Filenames");
TAP_ETCINFO(__DATE__);

#include "TSRCommander.inc"

// structure that holds the pointers to each detected event data item
typedef struct {
	char* ptrs[NUM_EVENT_DATA];
	byte  numPtrs;  // actual number that are pointing somewhere meaningful
	bool baseDataFound; // TRUE if data was found in the "normal" spot in the file, FALSE if not. Currently only used for subTitles
} TYPE_EventDataRecord;

// hardware types
typedef enum {
	HW_5000PVRt = 0,
	HW_MP = 1,
	HW_Procaster = 2,
	HW_5800 = 3,
	HW_None = 4,
	HW_5200 = 5
} TYPE_Hardware_Enum;
#define NUM_HARDWARE_TYPES 6

// languages
typedef enum {
	Lang_English = 0,
	Lang_Finnish = 1
} TYPE_Language_Enum;
#define NUM_LANGUAGE_TYPES 2

// Auto rec EPG fix modes
typedef enum {
	Mode_Off = 0,
	Mode_Smart = 1,
	Mode_Fixed10 = 2,
	Mode_Fixed20 = 3,
	Mode_Fixed30 = 4
} TYPE_Mode_Enum;
#define NUM_EPG_FIX_MODE_TYPES 5

// Auto file rename modes
typedef enum {
	Rename_Mode_Off = 0,
	Rename_Mode_On = 1,
	Rename_Mode_On_Subtitle = 2
} TYPE_Rename_Mode_Enum;
#define NUM_AUTO_RENAME_MODE_TYPES 3

// newfolder name modes
typedef enum {
	New_Folder_GROUP01 = 0,
	New_Folder_GRUPP01 = 1,
	New_Folder_GRUPPE01 = 2
} TYPE_New_Folder_Enum;
#define NUM_NEW_FOLDER_MODE_TYPES 3

// mod by Happy to get GMT offset automatically
#define pSysID          0xa3fffffa
dword                   SysID;
TYPE_ClockSetup         *ClockSetup;
static long 			EPGoffset = 0;
// state variables
static byte stirfBoxShowing = FALSE;
static bool configMenuShowing = FALSE;
static bool configOptionsChanged = FALSE;
static bool launchStirfBox = FALSE;
static int stirfBoxSelectedFile = 0;  //appropriate values are 0 to stirfBoxNumFiles - 1. Maintained by event handler.
static int stirfBoxNumFiles = 0; //actual number of items
static byte stirfBoxPageNum = 0;  //appropriate values are 0 to (stirfBoxNumFiles - 1)/PERPAGE...
static bool runStirfNow = FALSE;
static int subSelection[MAX_NUM_FILES]; //Stores the "ptrs" array index of the pointer to the selected suggested new filename for each file. 0 = no change. Appropriate array indices are 0 to stirfBoxNumFiles - 1.
static byte extInfoSelection[MAX_NUM_FILES]; //Stores the "ptrs" array index of the pointer to the selected suggested new extended info entry
static byte numWordsToTrimStart[MAX_NUM_FILES]; //Stores counters of how many words to trim off the start of each suggested file name
static byte numWordsToTrimEnd[MAX_NUM_FILES]; //Stores counters of how many words to trim off the end of each suggested file name
static int numFilesModified = 0;  // for status display after Stirfing
static int numFilesRenamed = 0;  // for status display after Stirfing
static int textScrollOffset = 0;
static int textScrollMaxOffset = 0;
static dword lastScrollTick = 0;
// data storage for the rec file info
static char fileNames[MAX_NUM_FILES][TS_FILE_NAME_SIZE]; //filenames of the files that are found to be suitable for stirf-ing
static int trimmedFileNameLen[MAX_NUM_FILES]; //stores the length of the trimmed file name (without time/date stamp or ".rec") for each file
static char subTitleData[MAX_NUM_FILES][SUBT_AREA_SIZE]; //copies of the subtitle info areas (whole, unformatted) found in the files
static char extInfoData[MAX_NUM_FILES][EXT_INFO_AREA_SIZE]; //copies of the extended info areas (whole, unformatted) found in the files
static TYPE_EventDataRecord titleRecords[MAX_NUM_FILES]; //these structures contain pointers into the subTitleData strings
static TYPE_EventDataRecord subTitleRecords[MAX_NUM_FILES]; //these structures contain pointers into the subTitleData strings
static TYPE_EventDataRecord extInfoRecords[MAX_NUM_FILES]; //these structures contain pointers into the extInfoData strings
static dword highlightColour, backgroundColour; //colours for text boxes. backgroundColour must match the bgrnd used in the stirf GD graphic
// Stirf Options
static bool trapIllegalChars = FALSE;
static TYPE_Mode_Enum recEPGfixMode = Mode_Off;
static TYPE_Hardware_Enum hardwareType = HW_5000PVRt;
static TYPE_Language_Enum language = Lang_English;
static int GMToffset15 = 32; // number of 15 minute intervals which the user's local time is offset from GMT. Default to +0800 for Perth, Australia
static TYPE_Rename_Mode_Enum autoRenameFiles = Rename_Mode_Off;
static TYPE_New_Folder_Enum newFolder = New_Folder_GROUP01;

// specialShowTitles stores the known titles of acronym/special-style shows (or any show that doesn't end in a lower case letter, really)
// specialShowTitles must have the names delimited by ';' and must end with a ';'
static char *specialShowTitles = NULL;
static char defaultSpecialShowTitles[] = "MDA;CSI: NY;CSI:NY;The 4400;JAG;24;American Dad!";
// OSD variables
static word stirfBoxRgn; //handle to OSD region
static word stirfBoxTempRgn; //handle to OSD region in memory (for full screen refreshes)
static word configMenuRegion;
static TYPE_Window configMenuWindow;
#define NUM_CONFIGMENU_ITEMS 9 // includes the "Exit Stirf" and "Next TAP" items

#define STIRFBOX_W  720
#define STIRFBOX_H  576
#define STIRFBOX_X  0
#define STIRFBOX_Y  0
#define CONFIG_W 350
#define CONFIG_H 180

#include "stirfStringFunctions.c"
#include "stirfGUIfunctions.c"
#include "stirfLib.c"
#include "recEPGfix.c"

void SetEPGoffset()
{
	switch (hardwareType)
	{
		case HW_5800: EPGoffset = 4;
			break;
		case HW_5200: EPGoffset = -4;
			break;
		default: EPGoffset = 0;
	}	
}
void SetFolderName()
{
	switch(newFolder)
	{
		case 0: strcpy(GROUP01, "GROUP 01");
			break;
		case 1: strcpy(GROUP01, "GRUPP 01");
			break;
		default: strcpy(GROUP01, "GRUPPE 01");
	}
}

// called before showStirfBox for the first time the user brings up
// the stirf window (i.e. not before a redraw etc)
void resetStirfBox( void ) {
	
	int i,j;
	
	stirfBoxSelectedFile = 0;
	stirfBoxPageNum = 0;
	stirfBoxNumFiles = 0;
	// reset all highlighting/boolean tags etc to 0
	memset ( subSelection, 0, sizeof subSelection);
	memset ( extInfoSelection, 0, sizeof extInfoSelection);
	memset ( fileNames, 0, sizeof fileNames);
	memset ( trimmedFileNameLen, 0, sizeof trimmedFileNameLen);
	memset ( subTitleData, 0, sizeof subTitleData);
	memset ( extInfoData, 0, sizeof extInfoData);
	memset ( numWordsToTrimEnd, 0, sizeof numWordsToTrimEnd);
	memset ( numWordsToTrimStart, 0, sizeof numWordsToTrimStart);
	
	for (i = 0; i < MAX_NUM_FILES; i++) {
		for (j = 0; j < NUM_EVENT_DATA; j++) {
			titleRecords[i].ptrs[j] = NULL;
		}
		titleRecords[i].numPtrs = 0; // actual number
		titleRecords[i].baseDataFound = FALSE;
	}
	for (i = 0; i < MAX_NUM_FILES; i++) {
		for (j = 0; j < NUM_EVENT_DATA; j++) {
			subTitleRecords[i].ptrs[j] = NULL;
		}
		subTitleRecords[i].numPtrs = 0; // actual number
		subTitleRecords[i].baseDataFound = FALSE;
	}
	for (i = 0; i < MAX_NUM_FILES; i++) {
		for (j = 0; j < NUM_EVENT_DATA; j++) {
			extInfoRecords[i].ptrs[j] = NULL;
		}
		extInfoRecords[i].numPtrs = 0; // actual number
		extInfoRecords[i].baseDataFound = FALSE;
	}

}

//////////////////////////////getFileList()////////////////////
//
// this is the code that fills in the file list data for the current
// directory (ie. your directory changing must have been done already).
// It also fills in the EventDataRecords for the subtitles and extended
// info slots that it finds in the files.
void getFileList( void ) {
	
	TYPE_File currentFile;
	TYPE_File* openedFile;
	int fileIndex = 0;
	int eventNameLength = 0;
	int numFilesAndDirs = 0;
	int rawSubtitleLength = 0;
	dword searchPosn = 0;
	int numSubTitlesFound = 0;
	int numExtInfosFound = 0;
	dword extInfoLen = 0;
	int foundNext = 1;
	char readBuffer[READ_BUFFER_SIZE];
	int trimmedLen = 0;
	char recordingFileName1[TS_FILE_NAME_SIZE];
	char recordingFileName2[TS_FILE_NAME_SIZE];
	TYPE_RecInfo recInf;
	
	recordingFileName1[0] = '\0';
	recordingFileName2[0] = '\0';
	memset(readBuffer, 0, sizeof readBuffer);
	stirfBoxNumFiles = 0;

	numFilesAndDirs = TAP_Hdd_FindFirst( &currentFile );
		
	if (numFilesAndDirs > 0) {
		
		// Check  current recording status (but only if we are in the datafiles directory, else leave the strings "\0")
		if ( strlen(uGbl.uCurOnScreenuDir.path) <= 10 ) {
			if (TAP_Hdd_GetRecInfo (0, &recInf)) {
				if ( recInf.recType == RECTYPE_Normal) {
					strcpy(recordingFileName1,recInf.fileName);
				}
			}
			if (TAP_Hdd_GetRecInfo (1, &recInf)) {
				if ( recInf.recType == RECTYPE_Normal) {
					strcpy(recordingFileName2,recInf.fileName);
				}
			}
		}

		for (fileIndex = 0; (stirfBoxNumFiles < MAX_NUM_FILES) && (fileIndex < (numFilesAndDirs)); fileIndex++) {
//				TAP_Print("1. %s %d,%d\r\n",currentFile.name,fileIndex,numFilesAndDirs);
			if ( (currentFile.attr == ATTR_TS) && (IsFileRec(currentFile.name)) && (currentFile.size > 1500) && (strcmp(currentFile.name,recordingFileName1) != 0) && (strcmp(currentFile.name,recordingFileName2) != 0) ) {
//			if ( (IsFileRec(currentFile.name)) && (strcmp(currentFile.name,recordingFileName1) != 0) && (strcmp(currentFile.name,recordingFileName2) != 0) )  //need curly bracket
//				TAP_Print("2. %s %d,%d\r\n",currentFile.name,fileIndex,numFilesAndDirs);
				if (openedFile = TAP_Hdd_Fopen(currentFile.name)) { // add subtitle conditions in here
					numSubTitlesFound = 0;
					numExtInfosFound = 0;
					// The 5800's rec file headers have 4 extra bytes before the event data, compared to Oz toppies
					TAP_Hdd_Fseek(openedFile, 85 + EPGoffset, SEEK_SET);
					// Read the event name length, plus one spare byte in-between, plus 257 bytes of event text.
					TAP_Hdd_Fread(readBuffer, READ_BUFFER_SIZE, 1, openedFile);
					// Read 1024 bytes of Extended info section from offset 368 (contrary to TopField documentation)
					// note again that the 5800's rec file headers have 4 extra bytes before the event data, compared to Oz toppies
					TAP_Hdd_Fseek(openedFile, 368 + EPGoffset, SEEK_SET);
					TAP_Hdd_Fread(extInfoData[stirfBoxNumFiles], EXT_INFO_AREA_SIZE, 1, openedFile);
					// Close the file
					TAP_Hdd_Fclose(openedFile);
					readBuffer[READ_BUFFER_SIZE-1] = 0; //just in case
					extInfoData[stirfBoxNumFiles][EXT_INFO_AREA_SIZE-1] = 0; //just in case. Also this is used later as a means of allowing the user to choose a blank entry
					//
					// START LOOKING FOR SUBTITLES
					//					
					eventNameLength = readBuffer[0];
					// The first two bytes in readBuffer are not to do with the text. The next 'eventNameLength' bytes
					// are a copy of the program name (not subtitle). After that is the subtitle proper.
					memcpy(subTitleData[stirfBoxNumFiles], (readBuffer+2), SUBT_AREA_SIZE );
					subTitleData[stirfBoxNumFiles][SUBT_AREA_SIZE-1] = 0; //just in case
					rawSubtitleLength = strlen(readBuffer+2+eventNameLength); //on Procaster, strlen of the 0x05 or '\0'
					// Format the subtitle as we wish... to be implemented??
					// If subtitle is useable... (we don't want the genre data, which starts with '['
					if ( (strlen(subTitleData[stirfBoxNumFiles] + eventNameLength) > 1) ) {
															//&& (subTitleData[stirfBoxNumFiles][eventNameLength] != '[')
						// Procaster has one 0x05 byte at the start that we want to avoid
						titleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound] = subTitleData[stirfBoxNumFiles] + ((hardwareType==HW_Procaster) ? 1:0);
						subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound] = subTitleData[stirfBoxNumFiles] + eventNameLength;
						// On the Procaster the last line would usually leave the subtitle pointer pointing at another 0x05 byte
						if ((hardwareType == HW_Procaster) && (*(subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound]) == 5 )) {
							subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound] = subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound] + 1;
						}
						subTitleRecords[stirfBoxNumFiles].baseDataFound = TRUE;
						numSubTitlesFound++;
					} else {
						subTitleRecords[stirfBoxNumFiles].baseDataFound = FALSE;
					}
					// now look for more subtitles in the spare space set aside in the .rec file
					// they are broadly delimited by byte sequence {0,64,0} and occur in sequences of
					// title,subtitle,0,garbage...,0,64,0,...zeros...,Title,Subtitle,0,garbage...,0,64,0,...zeros... etc
					// can't use string functions due to all the zeros (=null character). would use memmem but it isn't available :-(
					// look for 0,64,0 starting after the first set of data we already harvested:
					searchPosn = eventNameLength+rawSubtitleLength;
					// store the trimmed length temporarily for comparison against show titles
					trimmedLen = getTrimmedFileNameLength(currentFile.name);
					//loop
					while ( (searchPosn < (SUBT_AREA_SIZE-3)) && (numSubTitlesFound < NUM_EVENT_DATA) ) {
						while ( ( searchPosn < (SUBT_AREA_SIZE-2) ) && (subTitleData[stirfBoxNumFiles][searchPosn] != 64) ) {
							searchPosn++;
						}
						if ( ( searchPosn < (SUBT_AREA_SIZE-2) ) && (subTitleData[stirfBoxNumFiles][searchPosn-1] == 0) && (subTitleData[stirfBoxNumFiles][searchPosn+1] == 0) ) {
							//we found a delimiter. Move past it and past the zeros/crap trailing it
							searchPosn++;
							while ( ( searchPosn < (SUBT_AREA_SIZE-2) ) && (subTitleData[stirfBoxNumFiles][searchPosn] < 33) ) {
								searchPosn++;
							}
							if ( searchPosn != (SUBT_AREA_SIZE-2) ) {
								// searchPosn is now at the start of the title (not subtitle) of the next event
								titleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound] = &subTitleData[stirfBoxNumFiles][searchPosn];
								// In case we can't split the title and subtitle, store the whole lot now
								subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound] = &subTitleData[stirfBoxNumFiles][searchPosn];
								// Try to match title to trimmed file name
								if ( strncasecmp(titleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound], currentFile.name, trimmedLen) == 0) {
									// Then we matched the show title with the file name
									subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound] += trimmedLen;
									// Procaster sometimes separates title and subtitle with 0x05 byte
									if ( (hardwareType == HW_Procaster) && ( *(subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound]) == 5) ) {
										subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound] += 1;
									}									
									searchPosn += ( trimmedLen + ((hardwareType == HW_Procaster) ? 1:0) );
								} else { // try other methods of finding title length
									// Try to find cut point between title and subtitle by the fact there's no gap, so there
									// will be (barring acronym/special-named shows) a lower case then upper case with no space between:
									while ( ( searchPosn < (SUBT_AREA_SIZE-2) ) && ( !isLowerUpper(subTitleData[stirfBoxNumFiles],searchPosn) ) && (subTitleData[stirfBoxNumFiles][searchPosn] != 0) ) {
										searchPosn++;
									}
									// Procaster sometimes separates title and subtitle with 0x05 byte
									if ((hardwareType == HW_Procaster) && (strchr(titleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound],5) != NULL)) {
										subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound] = strchr(titleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound],5) + 1;
									} else if ( isLowerUpper(subTitleData[stirfBoxNumFiles],searchPosn) ) {
										// now we've found a capital after a lower case, take the rest as the subtitle
										// (replace previous stored pointer position now we have narrowed it down)
										subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound] = &subTitleData[stirfBoxNumFiles][searchPosn];
									} else { // we didn't find a lower-upper case pair
										// so let's see if we can find the show name from a list of known acronym/special type shows
										subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound] += getSpecialTitleLength(subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound]);
									}
								}
								// if we didn't find a lowercase-uppercase pair, nor an acronym/special style title, nor a show
								// title that matched the trimmed file name, then the pointer we stored before we looked
								// for those two remains as the result. Too bad.
								// Now check the subtitle we found was OK
								if ( strlen(subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound]) > 1) {
									// if it's just genre info (starts with "[" or " [") then don't record it
									if ( (subTitleData[stirfBoxNumFiles][searchPosn] != '[') && ((subTitleData[stirfBoxNumFiles][searchPosn] != ' ') || (subTitleData[stirfBoxNumFiles][searchPosn+1] != '[')) ) {
										//check it isn't a duplicate before recording it
										if (numSubTitlesFound > 0) {
											if ( strcmp(subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound],subTitleRecords[stirfBoxNumFiles].ptrs[numSubTitlesFound - 1]) != 0) {
												numSubTitlesFound++;
											}
										} else { // this was the first one we found so couldn't have been a duplicate
											numSubTitlesFound++;
										}
									}
								}
							} // else there was a delimiter near the end. No result.
						} else if (subTitleData[stirfBoxNumFiles][searchPosn] == 64) {
							searchPosn++;
						} // else we are near the end
					} //end of first while loop for subtitle data
					// so did we achieve anything?
					if ( numSubTitlesFound > 0 ) {
						// then we will record the info for this file.
						strcpy(fileNames[stirfBoxNumFiles],currentFile.name);
						trimmedFileNameLen[stirfBoxNumFiles] = trimmedLen;
						subTitleRecords[stirfBoxNumFiles].numPtrs = numSubTitlesFound;
						titleRecords[stirfBoxNumFiles].numPtrs = numSubTitlesFound;
						//
						// START LOOKING FOR EXTENDED INFO DATA
						//
						if (extInfoData[stirfBoxNumFiles][0] != '\0') { //then there is extended info available in the main slot
							extInfoRecords[stirfBoxNumFiles].ptrs[0] = extInfoData[stirfBoxNumFiles];
							numExtInfosFound = 1;
							extInfoRecords[stirfBoxNumFiles].baseDataFound = TRUE;
							searchPosn = strlen(extInfoData[stirfBoxNumFiles]) + 1;
						} else {
							// store the empty string in case we find non-empty ones later. If we don't, we'll undo this later
							extInfoRecords[stirfBoxNumFiles].ptrs[0] = extInfoData[stirfBoxNumFiles];
							numExtInfosFound = 1;
							extInfoRecords[stirfBoxNumFiles].baseDataFound = FALSE;
							searchPosn = 1;
						}
						//loop
						while ( (searchPosn < (EXT_INFO_AREA_SIZE-3)) && (numExtInfosFound < NUM_EVENT_DATA) ) {
							while ( ( searchPosn < (EXT_INFO_AREA_SIZE-2) ) && (extInfoData[stirfBoxNumFiles][searchPosn] != 64) ) {
								searchPosn++;
							}
							if ( ( searchPosn < (EXT_INFO_AREA_SIZE-2) ) && (extInfoData[stirfBoxNumFiles][searchPosn-1] == 0) && (extInfoData[stirfBoxNumFiles][searchPosn+1] == 0) ) {
								//we found a delimiter. Move past it and past the zeros trailing it
								searchPosn++;
								while ( ( searchPosn < (EXT_INFO_AREA_SIZE-2) ) && (extInfoData[stirfBoxNumFiles][searchPosn] < 33) ) {
									searchPosn++;
								}
								if ( searchPosn != (EXT_INFO_AREA_SIZE-2) ) {
									// searchPosn is now at the start of the extended info entry
									extInfoLen = strlen(&extInfoData[stirfBoxNumFiles][searchPosn]);
									if ( extInfoLen > 1 ) {
										if (numExtInfosFound == 0) { // then add it
											extInfoRecords[stirfBoxNumFiles].ptrs[numExtInfosFound] = &extInfoData[stirfBoxNumFiles][searchPosn];
											numExtInfosFound++;
										} else if ( !isStartingSubStr(&extInfoData[stirfBoxNumFiles][searchPosn],extInfoRecords[stirfBoxNumFiles].ptrs[(numExtInfosFound - 1)]) ) {
											// Then check to see it isn't a duplicate or a truncated version of the previous one before recording its position
											extInfoRecords[stirfBoxNumFiles].ptrs[numExtInfosFound] = &extInfoData[stirfBoxNumFiles][searchPosn];
											numExtInfosFound++;
										}
										searchPosn += extInfoLen;
									} else { // a problem has occured, move past it
										searchPosn++;
									}
								} // else we are near the end, let the while loop close out
							} else { // we found a 64 byte that wasn't a proper delimiter
								searchPosn++;
							}
						}
						// Finished looking for extended info entries. If a blank was not found at the start, then (space permitting)
						//  add another pointer (to a byte we know is '\0' from above) to allow the user to blank the extended info
						//  section if they want to.
						if ( (!extInfoRecords[stirfBoxNumFiles].baseDataFound) && (numExtInfosFound == 1) ) {
							// Then there were no extended info entries found at all
							numExtInfosFound = 0;
							extInfoRecords[stirfBoxNumFiles].ptrs[0] = NULL;
						} else if ( (extInfoRecords[stirfBoxNumFiles].baseDataFound) && (numExtInfosFound < (NUM_EVENT_DATA-1)) ) {
							extInfoRecords[stirfBoxNumFiles].ptrs[numExtInfosFound] = &extInfoData[stirfBoxNumFiles][EXT_INFO_AREA_SIZE-1];
							numExtInfosFound++;
						}
						extInfoRecords[stirfBoxNumFiles].numPtrs = numExtInfosFound;
						// finally, increment the counter (which we will use as an array index for the next file)
						// this must be after all the looking for subtitles/extended info business
						stirfBoxNumFiles++;
					} else { // we didn't find any subtitles
						//stirfBoxNumFiles stays the same and we'll "overwrite" with the next file
//				       TAP_Print("%s no subt found\r\n",currentFile.name);
						memset(subTitleData[stirfBoxNumFiles], 0, SUBT_AREA_SIZE);
						memset(extInfoData[stirfBoxNumFiles], 0, EXT_INFO_AREA_SIZE);
					}
					//clean up
					memset(readBuffer, 0, sizeof readBuffer);
				} else { //TAP_Hdd_Fopen failed
					switch (language) {
						case Lang_Finnish:
							NoteMenuCreate ( "STIRF virhe!", "Tiedostoa ei voitu avata", 300 ); 
							break;
						case Lang_English:
						default:
							NoteMenuCreate ( "STIRF Error!", "File could not be opened", 300 ); 
							break;
					}
					NoteMenuDelete();
				}
			}
			foundNext = TAP_Hdd_FindNext( &currentFile );
			if (foundNext == 0) {
				switch (language) {
					case Lang_Finnish:
						NoteMenuCreate ( "STIRF Error! Seuraavaa", "tiedostoa ei lytynyt", 300 );
						break;
					case Lang_English:
					default:
						NoteMenuCreate ( "STIRF Error!", "Next file could not be found.", 300 );
						break;
				}
				NoteMenuDelete();
				fileIndex = numFilesAndDirs; //bail out of the for loop as some problem has happened with the HDD access
			}
		}
	}
}

// this function actually does the file modifications after the GUI has
// been shown, the user has made their selections, and has pressed OK.
void runStirf( void ) {
	
	int fileNum, ptrNum, i;
	char buff[EXT_INFO_AREA_SIZE]; //needs to stay longer than READ_BUFFER_SIZE to account for stretching or Procaster format into 5000PVRt format
//	char displayMsg[40];
	char newFileName[TS_FILE_NAME_SIZE];
	char* tempPtr = NULL;
	TYPE_File* openedFile;
	bool succeeded = FALSE;
	word extLen = 0;
	byte extInfoLen[2];
	
	extInfoLen[0] = 0; extInfoLen[1] = 0;
//	memset(displayMsg, 0, sizeof displayMsg);
	memset(buff, 0, EXT_INFO_AREA_SIZE);

	switch (language) {
		case Lang_Finnish:
			NoteMenuCreate ( "Stirf kynniss!", "Tiedostoja pivitetn...", 200 );
			break;
		case Lang_English:
		default:
			NoteMenuCreate ( "STIRF Running!", "Updating files...", 200 );
			break;
	}
	NoteMenuDelete();

	numFilesModified = 0;
	numFilesRenamed = 0;

	for (fileNum = 0; fileNum < stirfBoxNumFiles; fileNum++) {
		// check if we must open the file
		if ( (ABS(subSelection[fileNum]) > ( (subTitleRecords[fileNum].baseDataFound) ? (1):(0) )) || (extInfoSelection[fileNum] != 0) ) {
			if ( openedFile = TAP_Hdd_Fopen(fileNames[fileNum]))  {
				// check if we must fix the title in the file's event data
				if ( ABS(subSelection[fileNum]) > ( (subTitleRecords[fileNum].baseDataFound) ? (1):(0) ) ) {
					ptrNum = (ABS(subSelection[fileNum]) - 1) % subTitleRecords[fileNum].numPtrs;
					// build the 259 byte area holding the event title length, spare byte, then 257 bytes of title/subtitle area
					buff[0] = subTitleRecords[fileNum].ptrs[ptrNum] - (titleRecords[fileNum].ptrs[ptrNum]);
					// do some just-in-case error checking
					if ( (buff[0] < 0) || buff[0] > 120 ) {
						// then we really just have to have a guess
						buff[0] = min(30,max(0,strlen(subTitleRecords[fileNum].ptrs[ptrNum])/2));
					}
					// don't really know how to deal with the parental rating. have to set it to zero
					buff[1] = 0;
					// if we wanted no change: memcpy(buff+2,subTitleData[fileNum],SUBT_AREA_SIZE);
					// Build a new event data region with the old entries re-ordered.
					
					/* This code has been replaced with a call to constructTitleDescriptionArea(...)
					// Must add in the 0x05 byte for the Procaster
					buff[2] = 5;
					// Over-write the 0x05 if not Procaster
					strcpy( buff + 2 +((hardwareType==HW_Procaster) ? 1:0), titleRecords[fileNum].ptrs[ptrNum] );
					tempPtr = buff + 2 + strlen(buff+2);
					if ( subTitleRecords[fileNum].numPtrs > 1) {
						// save the other data we found
						for (i = 0; i < subTitleRecords[fileNum].numPtrs; i++) {
							if ( i != ptrNum ) { //i.e. don't duplicate
								// replicate the Toppy's structure here, with zeros up to a 64 ('@') which is at a multiple of 16
								tempPtr = buff - 6 + 16*(((tempPtr + 10 - buff + 5) / 16) + 1);
								*tempPtr = '@';
								tempPtr += 7;
								// tempPtr now points to the 'possibly the rating' byte. leave at zero for now,
								// except for the Procaster where it seems to always be 0x05
								if (hardwareType == HW_Procaster) *tempPtr = 5;
								tempPtr += 1; // move to the start of the text field
								strncat(tempPtr, titleRecords[fileNum].ptrs[i], READ_BUFFER_SIZE - (tempPtr - buff));
								tempPtr += strlen(tempPtr);
							}
						}
					}
					*/
					constructTitleDescriptionArea(buff+2, titleRecords[fileNum].ptrs, subTitleRecords[fileNum].numPtrs, ptrNum);
					// Now that we have copied the data in, catch the problem where we couldn't distinguish
					// the title from the subtitle (i.e. where the title and subtitle pointers were the same)
					if (buff[0] == 0) {
						// Then we couldn't distinguish the title from the subtitle.
						// Have to guess... or maybe delimit it by the genre info (if any)
						tempPtr = strchr(buff+2,'[');
						if ( (tempPtr != NULL) && (strchr(tempPtr,']') != NULL) ) {
							buff[0] = tempPtr - (buff + 2);
						} else {
							buff[0] = strlen(buff+2);
						}
					}
					TAP_Hdd_Fseek(openedFile, 85 + EPGoffset, SEEK_SET);
					TAP_Hdd_Fwrite(buff, READ_BUFFER_SIZE, 1, openedFile);
				}
				// check if we must fix the extended info in the file
				if ( (extInfoSelection[fileNum] != 0) && (extInfoRecords[fileNum].numPtrs > 0) ) {
					ptrNum = (extInfoSelection[fileNum]);
					memset(buff, 0, EXT_INFO_AREA_SIZE);
					// build a new extended data region with the old entries re-ordered
					constructExtendedInfoArea(buff, extInfoRecords[fileNum].ptrs, extInfoRecords[fileNum].numPtrs, ptrNum);
					
					/* This code replaced with call to constructExtendedInfoArea(...)
					strcpy( buff, extInfoRecords[fileNum].ptrs[ptrNum] );
					extLen = strlen(buff);
					tempPtr = buff + extLen;
					if ( extInfoRecords[fileNum].numPtrs > 1) {
						// save the other data we found
						for (i = 0; i < extInfoRecords[fileNum].numPtrs; i++) {
							if ( i != ptrNum ) { //i.e. don't duplicate
								// replicate the Toppy's structure here, with zeros up to a 64 ('@') which is at a multiple of 16
								tempPtr = buff - 8 + 16*(((tempPtr + 3 - buff + 9) / 16) + 1);
								*tempPtr = '@';
								tempPtr += 7;
								// tempPtr now points to a mystery byte, a bit like the 'possibly the rating' byte in the title area.
								// Leave this byte at zero for now
								//*tempPtr = something?;
								tempPtr += 1; // move to the start of the text field
								strncat(tempPtr, extInfoRecords[fileNum].ptrs[i], EXT_INFO_AREA_SIZE - (tempPtr - buff));
								tempPtr += strlen(tempPtr);
							}
						}
					}
					*/
					// Write the exteded info length into the header
					extLen = strlen(buff);
					extInfoLen[0] = extLen >> 8;
					extInfoLen[1] = extLen & 255;
					TAP_Hdd_Fseek(openedFile, 362 + EPGoffset, SEEK_SET);
					TAP_Hdd_Fwrite(extInfoLen, 2, 1, openedFile);
					// Write the extended info itself
					TAP_Hdd_Fseek(openedFile, 368 + EPGoffset, SEEK_SET);
					TAP_Hdd_Fwrite(buff, EXT_INFO_AREA_SIZE, 1, openedFile);
				}
				// close the file
				TAP_Hdd_Fclose(openedFile);
				numFilesModified ++;
			} else { // TAP_Hdd_Fopen failed
//				strncpy(displayMsg, fileNames[fileNum], 39);
//				displayMsg[39] = '\0';
//crashes the toppy				NoteMenuCreate ( "STIRF Error! Couldn't open file:", displayMsg, 300 );
//(I don't know why)				NoteMenuDelete();
			}
			memset(buff, 0, EXT_INFO_AREA_SIZE);
		}
		// check if we must rename it
		if ( (subSelection[fileNum] != 0) && (subTitleRecords[fileNum].numPtrs != 0) ) {
			memset(newFileName, 0, sizeof newFileName);
			makeSuggestedFileName(newFileName, fileNum, subSelection[fileNum], TRUE, trapIllegalChars, TRUE);
			if ( (strcmp(fileNames[fileNum], newFileName) != 0) && (TAP_Hdd_Exist ( fileNames[fileNum])) ) {
				succeeded = TAP_Hdd_Rename(fileNames[fileNum], newFileName);
	 			if (succeeded) { // API seems to return TRUE even when the file you're asking to rename doesn't exist...!!!!
					//numFilesRenamed++;
				} else {
					// show error message
//					strncpy(displayMsg, fileNames[fileNum], 39);
//					displayMsg[39] = '\0';
//crashes the toppy					NoteMenuCreate ( "STIRF Error! Couldn't rename file:", displayMsg, 300 );
//(I don't know why)					NoteMenuDelete();
				}
				numFilesRenamed++; // do this here due to the unreliable return value of TAP_Hdd_Rename
			}
		}

	} // end of loop through each file
}

// Saves the Stirf options in an text ini file in ProgramFiles directory
// Plenty of borrowing from Peter's JustEPG source here...
void	saveINIfile() {

	TYPE_File*	iniFile;
	char		*iniFileBuff = NULL;
	char 		tempBuff[120];

	iniFileBuff = (char *)TAP_MemAlloc(INI_MAX_SIZE);
	if (iniFileBuff == NULL) return;
	
	memset( iniFileBuff, 0, INI_MAX_SIZE );
	tempBuff[0] = '\0';

    // Doesn't work: TAP_Hdd_ChangeDir( "//" );
    ChangeDirRoot();
    TAP_Hdd_ChangeDir( "ProgramFiles" );

    if ( TAP_Hdd_Exist ( "eurostirf.ini" ) ) TAP_Hdd_Delete ( "eurostirf.ini" );

    TAP_Hdd_Create( "eurostirf.ini", ATTR_NORMAL );

	if ( iniFile = TAP_Hdd_Fopen ( "eurostirf.ini" ) ) {
		sprintf( &tempBuff, "***** %s *****\r\n***** Convert Illegal Characters in Filenames: 1 or 0\r\n", TAP_NAME );
		strcat( iniFileBuff, tempBuff );
		sprintf( &tempBuff,"TrapIllegalChars=%d\r\n",(trapIllegalChars ? 1:0 )); 
		strcat( iniFileBuff, tempBuff );
		sprintf( &tempBuff,"***** Automatic rec file EPG update mode (0 for off)\r\nRecEPGfix=%d\r\n",recEPGfixMode );
		strcat( iniFileBuff, tempBuff );
		sprintf( &tempBuff,"***** Hardware Type: 0=Standard Australian 5000PVRt, 1=MP,");
		strcat( iniFileBuff, tempBuff );
		sprintf( &tempBuff," 2=Procaster, 3=5800, 4=Non-specific, 5=5200C\r\nHardware=%d\r\n",hardwareType );
		strcat( iniFileBuff, tempBuff );
		sprintf( &tempBuff,"***** Language/Kieli: 0=English, 1=suomi\r\nLanguage=%d\r\n",language );
		strcat( iniFileBuff, tempBuff );
		sprintf( &tempBuff,"***** GMT Offset (in 15 minute increments: 32 = +0800)\r\nGMT Offset=%d\r\n",GMToffset15 );
		strcat( iniFileBuff, tempBuff );
		sprintf( &tempBuff,"***** Auto File Rename (0 for off, 1 for on, 2 for on-with-subtitle)\r\nAutoRename=%d\r\n",(autoRenameFiles) );
		strcat( iniFileBuff, tempBuff );
		sprintf( &tempBuff,"***** New Folder name (0 for GROUP 01, 1 for GRUPP 01, 2 for GRUPPE 01)\r\nNewFolder=%d\r\n",(newFolder) );
		strcat( iniFileBuff, tempBuff );
		// Adding more options may lead to a requirement to raise the sizeof(iniFileBuff)...
		//TAP_Hdd_Fwrite( iniFileBuff, sizeof( iniFileBuff ), 1, iniFile); // problems with this...? not sure why.
		TAP_Hdd_Fwrite( iniFileBuff, INI_MAX_SIZE, 1, iniFile);
		TAP_Hdd_Fclose( iniFile );
	}
	else {
		switch (language) {
			case Lang_Finnish:
				NoteMenuCreate ( "Virhe: eurostirf.ini tiedostoon", "ei voitu kirjoittaa", 300 );
				break;
			case Lang_English:
			default:
				NoteMenuCreate ( "Error: Could not", "open eurostirf.ini for writing", 300 );
				break;
		}
		NoteMenuDelete();
	}
	if (iniFileBuff != NULL) TAP_MemFree(iniFileBuff);
	SetEPGoffset();
	SetFolderName();
}

// Loads the Stirf options from the text ini file in ProgramFiles directory
// Plenty of borrowing from Peter's JustEPG source here...
void	loadINIfile() {

	TYPE_File*	iniFile;
	char		*readBuffer = NULL;
	char 		*readPosn = NULL;
	char 		*equalsPosn = NULL;
	char 		*newLinePosn = NULL;

	readBuffer = (char *)TAP_MemAlloc(INI_MAX_SIZE);
	if (readBuffer == NULL){
		return;
	} else {
		readPosn = readBuffer;
	}
	
	memset( readBuffer, 0, INI_MAX_SIZE);

	// Doesn't work: TAP_Hdd_ChangeDir( "//" );
    ChangeDirRoot();
	TAP_Hdd_ChangeDir( "ProgramFiles" );

	if ( iniFile = TAP_Hdd_Fopen ( "eurostirf.ini" ) ) {
		TAP_Hdd_Fseek(iniFile, 0, SEEK_SET);
		//read the file into a buffer
		TAP_Hdd_Fread(readBuffer, INI_MAX_SIZE, 1, iniFile);
		TAP_Hdd_Fclose(iniFile);
		// put two nulls at the end for safety
		readBuffer[INI_MAX_SIZE-1] = '\0';
		readBuffer[INI_MAX_SIZE-2] = '\0';
		
		// check for a too-short ini file
		if ( strlen ( readBuffer ) < 100 ) {
			saveINIfile();
			if (readBuffer != NULL) TAP_MemFree(readBuffer);
			return;
		}
		
		// start scanning through the file's contents
		while (*readPosn != '\0') {
			if (*readPosn == '*') {
				// then we skip this line because it starts with '*'
				readPosn = strchr(readPosn,'\n');
			} else if ( (*readPosn == '\r') || (*readPosn == '\n') ) {
				// skip the \r and \n characters (new line characters)
				while ( (*readPosn == '\r') || (*readPosn == '\n') ) readPosn++;
			} else {
				// we are at the start of a line of data (or a blank line if the user puts one in!)
				// Check if it's a valid ini line
				equalsPosn = strchr(readPosn,'=');
				newLinePosn = strchr(readPosn,'\r');
				if ( (equalsPosn == NULL) || (newLinePosn == NULL) ) {
					if (readBuffer != NULL) TAP_MemFree(readBuffer);
					return; //because there should be at least one somewhere if there is more valid data to load
				}
				// if the line is not ordered properly, go to the next line, otherwise check the line
				if ( ( newLinePosn < equalsPosn) || (equalsPosn == readPosn) || (newLinePosn == (equalsPosn+1)) ) {
					readPosn = newLinePosn;
				} else { // we have a good line
					if ( strncmp(readPosn,"TrapIllegalChars",16) == 0) {
						if ( (*(equalsPosn+1)) == '1' ) {
							trapIllegalChars = TRUE;
						} else {
							trapIllegalChars = FALSE;
						}
					} else if ( strncmp(readPosn,"RecEPGfix",9) == 0) {
						switch (*(equalsPosn+1)) {
							case '0':
							case '1':
							case '2':
							case '3':
							case '4':
								recEPGfixMode = (*(equalsPosn+1)) - 48; // convert from ASCII
								break;
							default: //bad number
								recEPGfixMode = Mode_Off;
								break;
						}
					} else if ( strncmp(readPosn,"Hardware",8) == 0) {
						switch (*(equalsPosn+1)) {
							case '0':
							case '1':
							case '2':
							case '3':
							case '4':
							case '5':
								hardwareType = (*(equalsPosn+1)) - 48; // convert from ASCII
								break;
							default: //bad number
								hardwareType = HW_None;
								break;
						}		
					} else if ( strncmp(readPosn,"Language",8) == 0) {
						switch (*(equalsPosn+1)) {
							case '0':
							case '1':
								language = (*(equalsPosn+1)) - 48; // convert from ASCII
								break;
							default: //bad number, default to English
								language = Lang_English;
								break;
						}
					} else if ( strncmp(readPosn,"GMT Offset",10) == 0) {
						GMToffset15 = GetInteger(equalsPosn+1);
						if (abs(GMToffset15) > 48) GMToffset15 = 0;
					} else if ( strncmp(readPosn,"AutoRename",10) == 0) {
						switch (*(equalsPosn+1)) {
							case '0':
							case '1':
							case '2':
								autoRenameFiles = (*(equalsPosn+1)) - 48; // convert from ASCII
								break;
							default: //bad number
								autoRenameFiles = Rename_Mode_Off;
								break;
						}
					} else if ( strncmp(readPosn,"NewFolder",9) == 0) {
						switch (*(equalsPosn+1)) {
							case '0':
							case '1':
							case '2':
								newFolder = (*(equalsPosn+1)) - 48; // convert from ASCII
								break;
							default: //bad number
								newFolder = New_Folder_GROUP01;
								break;
						}
					} // else we couldn't match the label in the line
					readPosn = strchr(readPosn,'\n');
				}
			}
			// check if the strchr search function returned NULL
			if (readPosn == NULL) {
				if (readBuffer != NULL) TAP_MemFree(readBuffer);
				return;
			}
		} // end of scanning while loop		
	} else { // error opening file, or didn't exist
//crashes the toppy, I don't know why		NoteMenuCreate ( "Couldn't open eurostirf.ini", "Creating new ini file", 300 ); NoteMenuDelete();
//                                          NoteMenuCreate ( "eurostirf.ini tiedosto ei auennut", "Luodaan uusi ini tiedosto", 300 ); NoteMenuDelete();
		saveINIfile();
		if (readBuffer != NULL) TAP_MemFree(readBuffer);
		return;
	}
}

dword TAP_EventHandler( word event, dword param1, dword param2 ) {
	
	dword mainState, subState;
	char sTemp[128];
	div_t divideResult;
	int numPages, i;
	dword tick = 0;

	TAP_GetState( &mainState, &subState );

	if( event == EVT_IDLE ) {
		
		TSRCommanderWork();
		
		if (recEPGfixMode != Mode_Off || autoRenameFiles != Rename_Mode_Off) {
			recEPGfixEventHandler();
		}
		
		if( (mainState == STATE_Normal) && (subState == SUBSTATE_Normal) ) {
		
			if ( launchStirfBox ) {
				launchStirfBox = FALSE;
				TAP_ExitNormal();
				createStirfBox();
				lastScrollTick = TAP_GetTick();
				return 0;
			}
		}
		
		tick = TAP_GetTick();
		
		// Handle scrolling text
		if ( stirfBoxShowing && (extInfoRecords[stirfBoxSelectedFile].numPtrs > 0 ) && (lastScrollTick > 0) ) {
			if ( (textScrollOffset == 0) && ( tick > (lastScrollTick + SCROLL_START_DELAY) ) && ((textScrollMaxOffset = strlen(extInfoRecords[stirfBoxSelectedFile].ptrs[(extInfoSelection[stirfBoxSelectedFile])]) - 10) > 50) ) {
				// Then start the scrolling
				textScrollOffset = 1;
				lastScrollTick = tick;
				drawExtendedInfo( stirfBoxRgn, stirfBoxSelectedFile );
			} else if ( (textScrollOffset > 0) && (tick > (lastScrollTick + SCROLL_LETTER_DELAY)) ) {
				textScrollOffset++;
				textScrollOffset = textScrollOffset % textScrollMaxOffset;
				lastScrollTick = tick;
				drawExtendedInfo( stirfBoxRgn, stirfBoxSelectedFile );
			}
		}
	
		if  ( ( uGbl.bActionDirLister ) && ( tick >= ( uGbl.dwWaitTime + 50 ) ) ) {

			//////////////// Code from Peter Gillespie's DirLister:
			//Move to current file. Remove GROUP 01 folder.
			//Use "" for root directory otherwise use directory name (eg "DataFiles\Cracker").
			//uGbl.bDirListActive = FALSE;
			uGbl.bActionDirLister = FALSE;		
			DirLister ( "DataFiles" );
			// Only one GROUP 01 to be found because we put it there 
			// But you must start in a datafiles dir to get a result...
			if ( strcmp ( uGbl.uCurOnScreenuDir.path , "" ) != 0 ) {
				uGbl.bGroup01Flag = FALSE;
				ChangeDirRoot();
		 		ChangeDir (uGbl.uCurOnScreenuDir.path );
				// Ensure user is in the DataFiles sub-directory
				memset( sTemp, 0, sizeof sTemp );
				strncpy( sTemp, uGbl.uCurOnScreenuDir.path, 9 );
				lower ( sTemp );
				if ( strcmp ( sTemp, "datafiles" ) == 0 ) {
					//DoRename (); (This is Peter's call to dirLister's renaming routine. I replace it.
					// We have successfully got the dirList, now quit the file list and
					// set the stirf box to show up next time we get an idle cycle
					resetStirfBox();
					getFileList();
					if (stirfBoxNumFiles == 0) {
						switch (language) {
							case Lang_Finnish:
								NoteMenuCreate ( "Stirf", "Sopivia tiedostoja ei lytynyt", 300 ); 
								break;
							case Lang_English:
							default:
								NoteMenuCreate ( "Stirf", "Didn't find any suitable files", 300 );
								break;
						}
						NoteMenuDelete();
					} else {
						launchStirfBox = TRUE;
					}
					if ( TAP_Hdd_Exist ( GROUP01 ) ) {
						TAP_Hdd_Delete ( GROUP01 );
					}
					return 0;
				} else {
					switch (language) {
						case Lang_Finnish:
							NoteMenuCreate ( "Toimii vain", "DataFiles alihakemistoissa", 300 );
							break;
						case Lang_English:
						default:
							NoteMenuCreate ( "Error! You must be in or", "under the DataFiles directory", 300 );
							break;
					}
					NoteMenuDelete();
					launchStirfBox = FALSE;
					if ( TAP_Hdd_Exist ( GROUP01 ) ) {
						TAP_Hdd_Delete ( GROUP01 );
					}
					return 0;
				}			
			} else {
				switch (language) {
					case Lang_Finnish:
						NoteMenuCreate ( "Stirf virhe!", "Kuvaruutuvalikko ei ole avoinna", 300 );
						break;
					case Lang_English:
					default:
						NoteMenuCreate ( "Stirf Error!", "Onscreen directory not detected", 300 );
						break;
				}
				NoteMenuDelete();
				launchStirfBox = FALSE;
				return 0;
			}
		}
		/////////////// end of Peter's code
		// we have done everything we want to do with an IDLE event
		return 0;
	} else if ( event == EVT_KEY ) {
		
		// speed things up for everyone else if we don't need the key
		// if... (to be implemented)
		
		// Stop scrolling the ext. info text
		textScrollOffset = 0;
		lastScrollTick = 0;
		
		switch ( param1 ) {
			case RKEY_Recall :
				// Allow 5800 users to use the Recall button...
				if (hardwareType != HW_5800) return param1;
				// else carry on as if subtitle was pressed. i.e. no "break" statement
			case RKEY_Subt :
				if  ( !stirfBoxShowing && (mainState == STATE_FileList) && ( !(uGbl.bActionDirLister) ) ) {
					// User pressed Sub-title key when in the file list!
					// check first to see if there is a dialog (like the delete yes/no menu, or the keyboard) open over the file list
					if (isDialogOverFileList()) {
						return param1;
					}
					// Now start looking at the HDD...
					// Check first that there isn't a recording running - REMOVED AT VERSION 1.1
					/*if ( IsSomethingRec() == TRUE ) {
						switch (language) {
							case Lang_Finnish:
								NoteMenuCreate ( "Stirf Ohjelmaa ei voi", "kytt tallennuksen aikana", 300 );
								break;
							case Lang_English:
							default:
								NoteMenuCreate ( "Stirf", "is disabled while recording.", 300 );
								break;
						}
						NoteMenuDelete();
						return 0;
					}*/
					// Run DirLister and exit the toppy file list
					// Extensive use of code from Peter Gillespie's DirLister follows...
					switch (language) {
						case Lang_Finnish:
							NoteMenuCreate ( "Avataan tiedostolistaa...", "", 1 );
							break;
						case Lang_English:
						default:
							NoteMenuCreate ( "Getting file list...", "", 1 );
							break;
					}
					DirLister ( "DataFiles" );
					memset ( uGbl.uCurOnScreenuDir.path, 0, sizeof  uGbl.uCurOnScreenuDir.path );
					memset ( uGbl.uCurOnScreenuDir.name, 0, sizeof 	uGbl.uCurOnScreenuDir.name );
					NoteMenuDelete();
					// we need to start with no GROUP 01 folders anywhere
					if ( uGbl.bGroup01Flag == TRUE ) {
						strcpy ( uGbl.uCurOnScreenuDir.path, "Cancelled: Existing GROUP 01 folders" );
						strcpy ( uGbl.uCurOnScreenuDir.name, "Cancelled: Existing GROUP 01 folders" );
						switch (language) {
							case Lang_Finnish:
								NoteMenuCreate( "Toiminto peruttu. GROUP 01" , "hakemisto on jo olemassa" , 300 );
								break;
							case Lang_English:
							default:
								NoteMenuCreate( "Operation Cancelled." , "Existing GROUP 01 folders." , 300 );
								break;
						}
						NoteMenuDelete();
						uGbl.iDirFinalCnt = 0;
						//uGbl.bDirListActive = FALSE;
						uGbl.bActionDirLister = FALSE;
						uGbl.bGroup01Flag = FALSE;
						return 0;
					} else { // success!
 						//uGbl.bDirListActive = TRUE;
						uGbl.bActionDirLister = TRUE;
						//"press" the green key to create a GROUP 01 marker directory
						TAP_GenerateEvent ( EVT_KEY, RKEY_F2, RKEY_F2 );
						// leave the file list
						switch (hardwareType) {
							case HW_5800:
								// The 5800 needs special treatment here due to its stupid key mapping or something
								TAP_GenerateEvent( EVT_KEY, RKEY_PlayList, RKEY_PlayList );
								break;
							default:
								TAP_GenerateEvent( EVT_KEY, RKEY_Exit, RKEY_Exit );
								break;
						}
						uGbl.dwWaitTime = TAP_GetTick();
						return 0;
					}
				} else {
					return param1;
				}
				break;
			case RKEY_F2: //green key is page up
				if ( stirfBoxShowing ) {
					if (stirfBoxNumFiles > STIRFBOXITEMSPERPAGE) {
						// move to same posn on prev page
						stirfBoxSelectedFile = stirfBoxSelectedFile - STIRFBOXITEMSPERPAGE;
						if (stirfBoxSelectedFile <= -1) {
							stirfBoxSelectedFile = stirfBoxNumFiles - 1;
							refreshStirfBox();
						} else {
							drawAllTextAndHighlight( stirfBoxRgn );
						}
					}
					lastScrollTick = TAP_GetTick();
					return 0;
				} else {
					return param1;
				}
				break;
				
			case RKEY_F3: //yellow key is page down
				if ( stirfBoxShowing ) {
					if (stirfBoxNumFiles > STIRFBOXITEMSPERPAGE) {
						stirfBoxSelectedFile = stirfBoxSelectedFile + STIRFBOXITEMSPERPAGE;
						// work out how many pages there are
						numPages = (stirfBoxNumFiles-1) / STIRFBOXITEMSPERPAGE; // 0 = 1 page, 1 = 2 pages, etc
						// work out where we are
						if ( (stirfBoxSelectedFile / STIRFBOXITEMSPERPAGE) == numPages) { //then we're now on the last page
							if (stirfBoxSelectedFile >= stirfBoxNumFiles) {
								stirfBoxSelectedFile = stirfBoxNumFiles - 1;
							}
							refreshStirfBox();
							lastScrollTick = TAP_GetTick();
							return 0;
						} else if ( (stirfBoxSelectedFile / STIRFBOXITEMSPERPAGE) > numPages) {
							// then we've pressed page down from the last page
							deselectBox(stirfBoxRgn, stirfBoxSelectedFile % STIRFBOXITEMSPERPAGE);
							stirfBoxSelectedFile = 0;
							drawAllTextAndHighlight( stirfBoxRgn );
							lastScrollTick = TAP_GetTick();
							return 0;
						} else {
							drawAllTextAndHighlight( stirfBoxRgn );
							lastScrollTick = TAP_GetTick();
						}
					}
					return 0;
				} else {
					return param1;
				}
				break;
			case RKEY_Ok :
				if ( stirfBoxShowing ) {
					// user pressed OK button in the stirf file list.
					launchStirfBox = FALSE;
					// If the user has asked that >0 files are to be modified or renamed, set
					// the marker on to do the work.
					for (i = 0; i < stirfBoxNumFiles; i++) {
						if ( (subSelection[i] != 0) || (extInfoSelection[i] != 0) ) {
							runStirfNow = TRUE;
							// break out of the checking loop
							i = stirfBoxNumFiles;
						}
					}
					if ( runStirfNow ) {
						// do the business
						runStirfNow = FALSE;
						runStirf();
						hideStirfBox();
						resetStirfBox();
						switch (language) {
							case Lang_Finnish:
								sprintf(sTemp, "Tiedostot: %d muokattu, %d nimetty", numFilesModified, numFilesRenamed);
								sTemp[35] = '\0';
								NoteMenuCreate ( "Stirf on valmis!", sTemp, 300 );
								break;
							case Lang_English:
							default:
								sprintf(sTemp, "Modified %d file(s), renamed %d.", numFilesModified, numFilesRenamed);
								sTemp[35] = '\0';
								NoteMenuCreate ( "Stirf Complete!", sTemp, 300 );
								break;
						}
						NoteMenuDelete();
						TAP_EnterNormal();
						// Now show the file list to leave the user where he/she started
						TAP_GenerateEvent( EVT_KEY, RKEY_PlayList, RKEY_PlayList );
						return 0;	
					} else {
						// Possible upgrade: maybe tell them that nothing will be done?
						hideStirfBox();
						resetStirfBox();
						TAP_EnterNormal();
						// Now show the file list to leave the user where he/she started
						TAP_GenerateEvent( EVT_KEY, RKEY_PlayList, RKEY_PlayList );
						return 0;
					}
				} else if (configMenuShowing) {
					// toggle option if appropriate
					switch (i = TAP_Win_GetSelection( &configMenuWindow)) {
						case 0:
							trapIllegalChars = !trapIllegalChars;
							break;
						case 1:
							recEPGfixMode = (recEPGfixMode+1) % NUM_EPG_FIX_MODE_TYPES;
							break;
						case 2:
							hardwareType = (hardwareType+1) % NUM_HARDWARE_TYPES;
							break;
						case 3:
							language = (language+1) % NUM_LANGUAGE_TYPES;
							break;
						case 4:
							// do nothing - force user to choose either left or right to change GMT offset
							break;
						case 5:
							autoRenameFiles = (autoRenameFiles+1) % NUM_AUTO_RENAME_MODE_TYPES;
							break;
						case 6:
							newFolder = (newFolder+1) % NUM_NEW_FOLDER_MODE_TYPES;
							break;
						case 7: // exit TAP
							hideConfigMenu();
							if (configOptionsChanged) {
								saveINIfile();
								configOptionsChanged = FALSE;
							}
							configMenuShowing = FALSE;
							TAP_EnterNormal();
							TSRCommander.ExitTAP = TRUE;
							return 0;
							break;
						case 8: // pass Exit key to next TAP
							hideConfigMenu();
							if (configOptionsChanged) {
								saveINIfile();
								configOptionsChanged = FALSE;
							}
							configMenuShowing = FALSE;
							TAP_EnterNormal();
							return RKEY_Exit;
							break;
						default: // should be unreachable
							break;
					}
					configOptionsChanged = TRUE;
					refreshConfigMenu(i);
					return 0;
				}
				break;
			case RKEY_VolUp :
				if ( stirfBoxShowing ) {
					numWordsToTrimEnd[stirfBoxSelectedFile] = 0;
					numWordsToTrimStart[stirfBoxSelectedFile] = 0;
					// move down the list of suggested filenames for the selected file
					subSelection[stirfBoxSelectedFile] = subSelection[stirfBoxSelectedFile] + 1;
					if ( subSelection[stirfBoxSelectedFile] == ( (2 * (subTitleRecords[stirfBoxSelectedFile].numPtrs)) + 1) ) {
						subSelection[stirfBoxSelectedFile] = -(subTitleRecords[stirfBoxSelectedFile].numPtrs);
					}
					drawSuggestedFilenames( stirfBoxRgn, stirfBoxSelectedFile );
					// try to match the extended info with the subtitle selection
					if ( extInfoRecords[stirfBoxSelectedFile].numPtrs > 0 ) {
						extInfoSelection[stirfBoxSelectedFile] = min(extInfoRecords[stirfBoxSelectedFile].numPtrs - 1, max(0,((ABS(subSelection[stirfBoxSelectedFile]) - 1) % (subTitleRecords[stirfBoxSelectedFile].numPtrs))) );
						drawExtendedInfo( stirfBoxRgn, stirfBoxSelectedFile );
					}
					lastScrollTick = TAP_GetTick();
					return 0;
				} else if (configMenuShowing) {
					// toggle option if appropriate
					switch (i = TAP_Win_GetSelection( &configMenuWindow)) {
						case 0:
							trapIllegalChars = !trapIllegalChars;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
							break;
						case 1:
							recEPGfixMode = (recEPGfixMode+1) % NUM_EPG_FIX_MODE_TYPES;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
							break;
						case 2:
							hardwareType = (hardwareType+1) % NUM_HARDWARE_TYPES;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
							break;
						case 3:
							language = (language+1) % NUM_LANGUAGE_TYPES;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
							break;
						case 4:
							GMToffset15 = GMToffset15 + 1;
							if (GMToffset15 == 49) GMToffset15 = -48;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
						case 5:
							autoRenameFiles = (autoRenameFiles+1) % NUM_AUTO_RENAME_MODE_TYPES;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
							break;
						case 6:
							newFolder = (newFolder+1) % NUM_NEW_FOLDER_MODE_TYPES;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
							break;
						default:
							break;
					}
					return 0;
				} else {
					return param1;
				}
				break;
			case RKEY_VolDown :
				if ( stirfBoxShowing ) {
					numWordsToTrimEnd[stirfBoxSelectedFile] = 0;
					numWordsToTrimStart[stirfBoxSelectedFile] = 0;
					// move up the list of suggested filenames for the selected file
					subSelection[stirfBoxSelectedFile] = subSelection[stirfBoxSelectedFile] - 1;
					if ( subSelection[stirfBoxSelectedFile] == (-subTitleRecords[stirfBoxSelectedFile].numPtrs - 1) ) {
						subSelection[stirfBoxSelectedFile] = 2*(subTitleRecords[stirfBoxSelectedFile].numPtrs);
					}
					drawSuggestedFilenames( stirfBoxRgn, stirfBoxSelectedFile);
					// try to match the extended info with the subtitle selection
					if ( extInfoRecords[stirfBoxSelectedFile].numPtrs > 0 ) {
						extInfoSelection[stirfBoxSelectedFile] = min(extInfoRecords[stirfBoxSelectedFile].numPtrs - 1, max(0,((ABS(subSelection[stirfBoxSelectedFile]) - 1) % (subTitleRecords[stirfBoxSelectedFile].numPtrs))) );
						drawExtendedInfo( stirfBoxRgn, stirfBoxSelectedFile );
					}
					lastScrollTick = TAP_GetTick();
					return 0;
				} else if (configMenuShowing) {
					// toggle option if appropriate
					switch (i = TAP_Win_GetSelection( &configMenuWindow)) {
						case 0:
							trapIllegalChars = !trapIllegalChars;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
							break;
						case 1:
							recEPGfixMode = (recEPGfixMode+NUM_EPG_FIX_MODE_TYPES-1) % NUM_EPG_FIX_MODE_TYPES;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
							break;
						case 2:
							hardwareType = (hardwareType+NUM_HARDWARE_TYPES-1) % NUM_HARDWARE_TYPES;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
							break;
						case 3:
							language = (language+NUM_LANGUAGE_TYPES-1) % NUM_LANGUAGE_TYPES;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
							break;
						case 4:
							GMToffset15 = GMToffset15 - 1;
							if (GMToffset15 == -49) GMToffset15 = 48;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
						case 5:
							autoRenameFiles = (autoRenameFiles-1) % NUM_AUTO_RENAME_MODE_TYPES;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
							break;
						case 6:
							newFolder = (newFolder-1) % NUM_NEW_FOLDER_MODE_TYPES;
							configOptionsChanged = TRUE;
							refreshConfigMenu(i);
							break;
						default:
							break;
					}
					return 0;
				} else {
					return param1;
				}
				break;
			case RKEY_ChUp :
				if ( stirfBoxShowing ) {
					// go up one item in the stirfBox file list
					stirfBoxSelectedFile--;
					if (stirfBoxSelectedFile == -1) {
						//then we moved up to the last page
						stirfBoxSelectedFile = stirfBoxNumFiles - 1;
						refreshStirfBox();
						lastScrollTick = TAP_GetTick();
						return 0;
					} else if ( (stirfBoxSelectedFile % STIRFBOXITEMSPERPAGE) == (STIRFBOXITEMSPERPAGE-1) ) {
						//then we just moved up a page
						refreshStirfBox();
						lastScrollTick = TAP_GetTick();
						return 0;
					} else {
						//we just moved up one on the same page
						deselectBox( stirfBoxRgn, (stirfBoxSelectedFile+1) % STIRFBOXITEMSPERPAGE );
						selectBox( stirfBoxRgn, stirfBoxSelectedFile % STIRFBOXITEMSPERPAGE );
						lastScrollTick = TAP_GetTick();
						return 0;
					}
					return 0;
				} else if (configMenuShowing) {
					TAP_Win_Action ( &configMenuWindow, param1 );
					return 0;
				} else {
					return param1;
				}
				break;
			case RKEY_ChDown :
				if ( stirfBoxShowing ) {
					// go down one item in the stirfBox file list
					stirfBoxSelectedFile++;
					if (stirfBoxSelectedFile == stirfBoxNumFiles) {
						//then we moved down to the first page
						stirfBoxSelectedFile = 0;
						refreshStirfBox();
						lastScrollTick = TAP_GetTick();
						return 0;
					} else if ( (stirfBoxSelectedFile % STIRFBOXITEMSPERPAGE) == 0 ) {
						//then we just moved down a page
						refreshStirfBox();
						lastScrollTick = TAP_GetTick();
						return 0;
					} else {
						//we just moved down one on the same page
						deselectBox( stirfBoxRgn, (stirfBoxSelectedFile-1) % STIRFBOXITEMSPERPAGE );
						selectBox( stirfBoxRgn, stirfBoxSelectedFile % STIRFBOXITEMSPERPAGE );
						lastScrollTick = TAP_GetTick();
						return 0;
					}
					return 0;
				} else if (configMenuShowing) {
					TAP_Win_Action ( &configMenuWindow, param1 );
					return 0;
				} else {
					return param1;
				}
				break;
			case RKEY_Info:
				if ( stirfBoxShowing ) {
					// user wants to cycle through the extended info's that were found in the file
					if (extInfoRecords[stirfBoxSelectedFile].numPtrs > 1) {
						extInfoSelection[stirfBoxSelectedFile] = (extInfoSelection[stirfBoxSelectedFile] + 1) % (extInfoRecords[stirfBoxSelectedFile].numPtrs);
						drawExtendedInfo (stirfBoxRgn, stirfBoxSelectedFile);
					}
					lastScrollTick = TAP_GetTick();
					return 0;
				} else {
					return param1;
				}
				break;
			case RKEY_Forward:
				if ( stirfBoxShowing ) {
					numWordsToTrimEnd[stirfBoxSelectedFile] = (numWordsToTrimEnd[stirfBoxSelectedFile] + 1) % 100;
					drawSuggestedFilenames( stirfBoxRgn, stirfBoxSelectedFile);
					lastScrollTick = TAP_GetTick();
					return 0;
				} else {
					return param1;
				}
				break;
			case RKEY_Rewind:
				if ( stirfBoxShowing ) {
					numWordsToTrimStart[stirfBoxSelectedFile] = (numWordsToTrimStart[stirfBoxSelectedFile] + 1) % 100;
					drawSuggestedFilenames( stirfBoxRgn, stirfBoxSelectedFile);
					lastScrollTick = TAP_GetTick();
					return 0;
				} else {
					return param1;
				}
				break;
			case RKEY_Exit :
				if ( stirfBoxShowing && (mainState != STATE_FileList) ) {
					launchStirfBox = FALSE;
					hideStirfBox();
					resetStirfBox();
					TAP_EnterNormal();
					TAP_GenerateEvent( EVT_KEY, RKEY_PlayList, RKEY_PlayList );
					return 0;
				} else if (configMenuShowing) {
					hideConfigMenu();
					if (configOptionsChanged) {
						saveINIfile();
						configOptionsChanged = FALSE;
					}
					configMenuShowing = FALSE;
					TAP_EnterNormal();
				} /*else if (!configMenuShowing && !OsdActive(50,550) && ( mainState == STATE_Normal)  && ( subState == SUBSTATE_Normal ) ) {
					// show the config menu
					TAP_ExitNormal();
					createConfigMenu();
					configMenuShowing = TRUE;
					return 0;
				} */else {
					return param1;
				}
				break;
		}
		// if our window is showing, then we have trapped all the events we want to
		if ( stirfBoxShowing || configMenuShowing ) {
			return 0;
		}
	}
	return param1;
}

void TSRCommanderConfigDialog() {
	// show the config menu
	TAP_ExitNormal();
	createConfigMenu();
	configMenuShowing = TRUE;
}

bool TSRCommanderExitTAP (void) {
	runStirfNow = FALSE;
	uGbl.bActionDirLister = FALSE;
	launchStirfBox = FALSE;
	if (stirfBoxShowing) hideStirfBox();
	TAP_EnterNormal();
	return TRUE;
}

int TAP_Main(void) {
	
	TSRCommanderInit( 0, TRUE );
	
	// set some constants
	highlightColour = RGB(22,22,22); //Grey for the background of selected text
	backgroundColour = RGB(64,0,255); //Blue for the background of unselected text - saves redrawing backgrounds
	
    // restore pointer values in GD data structures...
    _stirfGd.data = _stirfCpm;
	_finnishhelpGd.data = _finnishhelpCpm;
	_stirfonborderGd.data = _stirfonborderCpm;
	_stirfoffborderGd.data = _stirfoffborderCpm;

	specialShowTitles = defaultSpecialShowTitles;
	loadINIfile();
	SetEPGoffset();
	SetFolderName();
    SysID = *(word*)pSysID;
    if (FlashInitialize (SysID))
    {
    	ClockSetup = (TYPE_ClockSetup*) (FlashGetBlockStartAddress (7) + 4);
//    	TAP_Print ("UTC Offset %d\n",ClockSetup->UTCOffset);
    	GMToffset15 = (int)ClockSetup->UTCOffset / 15;
	}
	resetStirfBox();
	// initialise the recEPGfix routine
	recEPGfixMain();
	
	return 1;
}

			
