/*
*
*  Copyright (c) 2005 Michael Stock. All Rights Reserved.
*
*  This file is part of Stirf.
*
*  Stirf is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  Stirf is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this software; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


void NoteMenuDelete() {
	TAP_Win_Delete( &uGbl.iNoteMenuWin );
	TAP_Osd_Delete (uGbl.iNoteMenuRgn);
	uGbl.iNoteMenuRgnOn = FALSE;
}

// draws the highlight around the selected box on the stirf screen
// boxNum is between 0 and 2
void selectBox( const word rgn, const int boxNum ) {
   	TAP_Osd_PutGd( rgn, (720-_stirfGd.width)/2 +22, (576-_stirfGd.height)/2 +40+ boxNum*132, &_stirfonborderGd, TRUE );
}

// draws the "un-highlight" around the specified (de-selected) box on the stirf screen
// boxNum is between 0 and 2
void deselectBox( word rgn, const int boxNum ) {
   	TAP_Osd_PutGd( rgn, (720-_stirfGd.width)/2 +22, (576-_stirfGd.height)/2 +40+ boxNum*132, &_stirfoffborderGd, TRUE );
}

// draws the appropriate three suggested filenames for the given file
// boxNum is implied by fileIndex
void drawSuggestedFilenames( const word rgn, const int fileIndex ) {
	
	char temp[TS_FILE_NAME_SIZE];
	int boxNum = fileIndex % STIRFBOXITEMSPERPAGE; // from 0 to 2

	//write the first alternative
	makeSuggestedFileName(temp, fileIndex, subSelection[fileIndex] - 1, FALSE, FALSE, FALSE);
	TAP_Osd_PutS( rgn, 55+55, ((576-_stirfGd.height)/2)+74+ boxNum*132, 633, temp, COLOR_White, backgroundColour, 0, FNT_Size_1622, 1, ALIGN_LEFT );
	//write the middle (selected) alternative
	makeSuggestedFileName(temp, fileIndex, subSelection[fileIndex], FALSE, FALSE, TRUE);
	TAP_Osd_PutS( rgn, 55+55, ((576-_stirfGd.height)/2)+96+ boxNum*132, 633, temp, COLOR_White, highlightColour, 0, FNT_Size_1622, 1, ALIGN_LEFT );
	//write the last alternative
	makeSuggestedFileName(temp, fileIndex, subSelection[fileIndex] + 1, FALSE, FALSE, FALSE);
	TAP_Osd_PutS( rgn, 55+55, ((576-_stirfGd.height)/2)+118+boxNum*132, 633, temp, COLOR_White, backgroundColour, 0, FNT_Size_1622, 1, ALIGN_LEFT );

}

// draws the appropriate extended info for the given file
// boxNum is implied by fileIndex
void drawExtendedInfo( const word rgn, const int fileIndex ) {

	int boxNum = fileIndex % STIRFBOXITEMSPERPAGE; // from 0 to 2
	
	if (extInfoRecords[fileIndex].numPtrs == 0 ) {
		// then no extended info entries were found in this file
		switch (language) {
			case Lang_Finnish:
				TAP_Osd_PutS( rgn, 55+60, ((576-_stirfGd.height)/2)+143+boxNum*132,631, "(Ei listietoja tiedostossa)", COLOR_White, backgroundColour, 0, FNT_Size_1419, 1, ALIGN_LEFT );
				break;
			case Lang_English:
			default:
				TAP_Osd_PutS( rgn, 55+60, ((576-_stirfGd.height)/2)+143+boxNum*132,631, "(No extended information entries found in file)", COLOR_White, backgroundColour, 0, FNT_Size_1419, 1, ALIGN_LEFT );
				break;
		}
		// Cancel scrolling for this text
		textScrollOffset = 0;
		lastScrollTick = 0;
	} else if ( *(extInfoRecords[fileIndex].ptrs[(extInfoSelection[fileIndex])]) == '\0' ) {
		switch (language) {
			case Lang_Finnish:
				TAP_Osd_PutS( rgn, 55+60, ((576-_stirfGd.height)/2)+143+boxNum*132,631, "(Poista listiedot tiedostosta)", COLOR_White, backgroundColour, 0, FNT_Size_1419, 1, ALIGN_LEFT );
				break;
			case Lang_English:
			default:
				TAP_Osd_PutS( rgn, 55+60, ((576-_stirfGd.height)/2)+143+boxNum*132,631, "(Blank the extended info section)", COLOR_White, backgroundColour, 0, FNT_Size_1419, 1, ALIGN_LEFT );
				break;
		}
		// Cancel scrolling for this text
		textScrollOffset = 0;
		lastScrollTick = 0;
	} else {
		TAP_Osd_PutS( rgn, 55+60, ((576-_stirfGd.height)/2)+143+boxNum*132,631, extInfoRecords[fileIndex].ptrs[(extInfoSelection[fileIndex])] + textScrollOffset, COLOR_White, backgroundColour, 0, FNT_Size_1419, 1, ALIGN_LEFT );
	}
	
}

// draws the (up to) three boxes' text (filename, suggested filenames)
// for the current page shown (as implied by stirfBoxSelectedFile).
// highlights the selected file's box.
void drawAllTextAndHighlight( const word rgn ) {
	
	int boxCounter, currentFile, stirfBoxPageNum;
	
	stirfBoxPageNum = stirfBoxSelectedFile / STIRFBOXITEMSPERPAGE; // 0 is the first page

	for (boxCounter = 0; boxCounter < STIRFBOXITEMSPERPAGE; boxCounter++) {
		currentFile = (STIRFBOXITEMSPERPAGE*stirfBoxPageNum) + boxCounter; // min possible = 0, max = stirfBoxNumFiles-1
		//write the filename		
		TAP_Osd_PutS( rgn, 55+38, ((576-_stirfGd.height)/2)+49+ boxCounter*132, 630, fileNames[currentFile], COLOR_White, backgroundColour, 0, FNT_Size_1622, 1, ALIGN_LEFT );
		drawSuggestedFilenames( rgn, currentFile );
		drawExtendedInfo( rgn, currentFile );
		// lastly: if we have just drawn the last file in the list then don't draw any more even if screen isn't full
		if ( currentFile == (stirfBoxNumFiles - 1) ) {
			//so break out of the "for" loop
			boxCounter = STIRFBOXITEMSPERPAGE;
		}
	}
	selectBox(rgn,stirfBoxSelectedFile % STIRFBOXITEMSPERPAGE);
	
}

// redraws the whole box in stirfBoxTempRgn and copies it to stirfBoxRgn
void refreshStirfBox( void ) {
	
	TAP_Osd_PutGd( stirfBoxTempRgn, (720-_stirfGd.width)/2 , (576-_stirfGd.height)/2, &_stirfGd, FALSE );
	drawAllTextAndHighlight( stirfBoxTempRgn );
	switch (language) {
		case Lang_Finnish:
			TAP_Osd_PutGd( stirfBoxTempRgn, (720-_stirfGd.width)/2 , (576-_stirfGd.height)/2 + 444, &_finnishhelpGd, FALSE );
			break;
		case Lang_English:
		default:
			// do nothing
			break;
	}
	TAP_Osd_Copy( stirfBoxTempRgn, stirfBoxRgn, 0, 0, 720, 576, 0, 0, FALSE );
}

// creates the regions and draws to them
void createStirfBox( void ) {
	
	stirfBoxRgn = TAP_Osd_Create( STIRFBOX_X, STIRFBOX_Y, STIRFBOX_W, STIRFBOX_H, 0, 0 );
	stirfBoxTempRgn = TAP_Osd_Create( STIRFBOX_X, STIRFBOX_Y, STIRFBOX_W, STIRFBOX_H, 0, OSD_Flag_MemRgn );
	refreshStirfBox();
	stirfBoxShowing = TRUE;
}

void hideStirfBox( void ) {
	
	TAP_Osd_Delete( stirfBoxRgn );
	TAP_Osd_Delete( stirfBoxTempRgn );
	stirfBoxShowing = FALSE;
}

// This function's code is from Peter Gillespie's JustEPG
bool 	OsdActive (int iStartingRow,int iEndingRow) {
	TYPE_OsdBaseInfo osdBI;
	word* pAddr[576];
	dword i, x, y;

	TAP_Osd_GetBaseInfo (&osdBI);	// retrieve current OSD
	for (i = iStartingRow; i < iEndingRow; i += 4)
	{
		pAddr[i] = osdBI.eAddr + 720 * i;   //even field
		//pAddr[i+1] = osdBI.oAddr + 720 * i;  //odd field
	}
	for (y = iStartingRow; y < iEndingRow; y += 4)	// scan screen-buffer, every 2nd line to save resources
	{
		for (x = 180; x < 300; x+=6)  //may as well only scan every 6 pixels to save time and resources
		{
			if (*(pAddr[y] + x) != 0) // return TRUE if value at any position
				return TRUE;
		}
	}
	return FALSE;
}

// only adds the menu items, according to current state variables
void populateConfigWindow (void) {
	
	char itemString[50];
	itemString[0] = '\0';
	
	if (trapIllegalChars) {
		switch (language) {
			case Lang_Finnish:
				TAP_Win_AddItem( &configMenuWindow, "Korjaa ongelmamerkit: Pll");
				break;
			case Lang_English:
			default:
				TAP_Win_AddItem( &configMenuWindow, "Trap Illegal Characters: On");		
				break;
		}
	} else {
		switch (language) {
			case Lang_Finnish:
				TAP_Win_AddItem( &configMenuWindow, "Korjaa onglemamerkit: Pois plt");
				break;
			case Lang_English:
			default:
				TAP_Win_AddItem( &configMenuWindow, "Trap Illegal Characters: Off");
				break;
		}
	}
	
	// Auto rec EPG fix modes
	switch (language) {
		case Lang_Finnish:
			switch (recEPGfixMode) {
				case Mode_Smart:
					TAP_Win_AddItem( &configMenuWindow, "EPG:n autokorjaus: Pll");
					break;
				case Mode_Fixed10:
					TAP_Win_AddItem( &configMenuWindow, "EPG:n autokorjaus: Pll (10%)");
					break;
				case Mode_Fixed20:
					TAP_Win_AddItem( &configMenuWindow, "EPG:n autokorjaus: Pll (20%)");
					break;
				case Mode_Fixed30:
					TAP_Win_AddItem( &configMenuWindow, "EPG:n autokorjaus: Pll (30%)");
					break;
				case Mode_Off:
				default:
					TAP_Win_AddItem( &configMenuWindow, "EPG:n autokorjaus: Pois plt");
					break;
			}
			break;
		case Lang_English:
		default:
			switch (recEPGfixMode) {
				case Mode_Smart:
					TAP_Win_AddItem( &configMenuWindow, "Auto .rec EPG Fix: On (Smart)");
					break;
				case Mode_Fixed10:
					TAP_Win_AddItem( &configMenuWindow, "Auto .rec EPG Fix: On (10%)");
					break;
				case Mode_Fixed20:
					TAP_Win_AddItem( &configMenuWindow, "Auto .rec EPG Fix: On (20%)");
					break;
				case Mode_Fixed30:
					TAP_Win_AddItem( &configMenuWindow, "Auto .rec EPG Fix: On (30%)");
					break;
				case Mode_Off:
				default:
					TAP_Win_AddItem( &configMenuWindow, "Auto .rec EPG Fix: Off");
					break;
			}
			break;
	}

	// Hardware types
	switch (language) {
		case Lang_Finnish:
			sprintf(itemString,"Laitteisto: ");
			break;
		case Lang_English:
		default:
			sprintf(itemString,"Hardware: ");
			break;
	}
	switch (hardwareType) {
		case HW_5000PVRt:
			strcat(itemString,"5000PVRt/5100t/5000PVRs");
			TAP_Win_AddItem( &configMenuWindow, itemString );
			break;
		case HW_MP:
			strcat(itemString,"MasterPiece");
			TAP_Win_AddItem( &configMenuWindow, itemString );
			break;
		case HW_Procaster:
			strcat(itemString,"Procaster");
			TAP_Win_AddItem( &configMenuWindow, itemString );
			break;
		case HW_5800:
			strcat(itemString,"5800");
			TAP_Win_AddItem( &configMenuWindow, itemString );
			break;
		case HW_None:
			strcat(itemString,"(Non-specific)");
			TAP_Win_AddItem( &configMenuWindow, itemString );
			break;
		case HW_5200:
			strcat(itemString,"5200c");
			TAP_Win_AddItem( &configMenuWindow, itemString );
			break;
		default: // bad number was assigned, default to 5000PVRt
			strcat(itemString,"5000PVRt");
			configOptionsChanged = TRUE;
			hardwareType = HW_5000PVRt;
			TAP_Win_AddItem( &configMenuWindow, itemString );
			break;
	}
	switch (language) {
		case Lang_English:
			TAP_Win_AddItem( &configMenuWindow, "Language: English");
			break;
		case Lang_Finnish:
			TAP_Win_AddItem( &configMenuWindow, "Kieli: Suomi");
			break;
		default: // bad number was assigned, default to English
			configOptionsChanged = TRUE;
			language = Lang_English;
			TAP_Win_AddItem( &configMenuWindow, "Language: English");
			break;
	}
	// GMT offset
	itemString[0] = '\0';
	sprintf(itemString, "GMT %s%02d:%02d", ((GMToffset15 >= 0) ? "+":"-"), (abs(GMToffset15) / 4), 15*(abs(GMToffset15) % 4));
	TAP_Win_AddItem( &configMenuWindow, itemString );
	// Auto file renaming
	switch (language) {
		case Lang_Finnish:
			switch (autoRenameFiles) {
				case Rename_Mode_On:
					TAP_Win_AddItem( &configMenuWindow, "Ksinajastetut tied.nimet: Korjaa");
					break;
				case Rename_Mode_On_Subtitle:
					TAP_Win_AddItem( &configMenuWindow, "Ksinajastetut tied.nimet: Korjaa ( - ...)");
					break;
				case Rename_Mode_Off:
				default:
					TAP_Win_AddItem( &configMenuWindow, "Ksinajastetut tied.nimet: Ei korj");
					break;
			}
			break;
		case Lang_English:
		default:
			switch (autoRenameFiles) {
				case Rename_Mode_On:
					TAP_Win_AddItem( &configMenuWindow, "Auto File Rename: On");
					break;
				case Rename_Mode_On_Subtitle:
					TAP_Win_AddItem( &configMenuWindow, "Auto File Rename: On + subtitle");
					break;
				case Rename_Mode_Off:
				default:
					TAP_Win_AddItem( &configMenuWindow, "Auto File Rename: Off");
					break;
			}
	}
	// New folder name
	switch (language) {
		case Lang_Finnish:
			switch (newFolder) {
				case New_Folder_GROUP01:
					TAP_Win_AddItem( &configMenuWindow, "New Folder name: GROUP 01");
					break;
				case New_Folder_GRUPP01:
					TAP_Win_AddItem( &configMenuWindow, "New Folder name: GRUPP 01");
					break;
				case New_Folder_GRUPPE01:
				default:
					TAP_Win_AddItem( &configMenuWindow, "New Folder name: GRUPPE 01");
					break;
			}
			break;
		case Lang_English:
		default:
			switch (newFolder) {
				case New_Folder_GROUP01:
					TAP_Win_AddItem( &configMenuWindow, "New Folder name: GROUP 01");
					break;
				case New_Folder_GRUPP01:
					TAP_Win_AddItem( &configMenuWindow, "New Folder name: GRUPP 01");
					break;
				case New_Folder_GRUPPE01:
				default:
					TAP_Win_AddItem( &configMenuWindow, "New Folder name: GRUPPE 01");
					break;
			}
	}
	// Exit and next
	switch (language) {
		case Lang_Finnish:
			TAP_Win_AddItem( &configMenuWindow,"Sulje Stirf");
			TAP_Win_AddItem( &configMenuWindow,"Seuraava TAP");
			break;
		case Lang_English:
		default:
			TAP_Win_AddItem( &configMenuWindow,"Exit Stirf");
			TAP_Win_AddItem( &configMenuWindow,"Next TAP");
			break;
	}
}

void createConfigMenu (void) {
	configMenuRegion = TAP_Osd_Create( 0, 0, 720, 576, 0, FALSE );
	TAP_Win_SetDefaultColor( &configMenuWindow );
	TAP_Win_Create( &configMenuWindow, configMenuRegion, ((720-CONFIG_W)/2), ((576-CONFIG_H)/2)-40, CONFIG_W, CONFIG_H, FALSE, FALSE );
	TAP_Win_SetTitle( &configMenuWindow, TAP_NAME, 0, FNT_Size_1622 );
	populateConfigWindow();
	TAP_Win_SetSelection( &configMenuWindow, NUM_CONFIGMENU_ITEMS-1 );
}

void refreshConfigMenu (int itemToSelect) {
	TAP_Win_Delete( &configMenuWindow );
	TAP_Win_SetDefaultColor( &configMenuWindow );
	TAP_Win_Create( &configMenuWindow, configMenuRegion, ((720-CONFIG_W)/2), ((576-CONFIG_H)/2)-40, CONFIG_W, CONFIG_H, FALSE, FALSE );
	TAP_Win_SetTitle( &configMenuWindow, TAP_NAME, 0, FNT_Size_1622 );
	populateConfigWindow();
	TAP_Win_SetSelection( &configMenuWindow, itemToSelect );
}

void hideConfigMenu (void) {
	TAP_Win_Delete( &configMenuWindow );
	TAP_Osd_Delete( configMenuRegion );
	configMenuShowing = FALSE;
}

//--------------------------------------- isDialogOverFileListMP --------------------------------
// Uses a different method (GetPixel) becuase of the fact the MP has 8 bit colour and doesn't
// allow you to detect the file list OSD via GetBaseInfo.
// Graciously provided by Andy Cullen, thanks very much Andy.
bool isDialogOverFileListMP(void) {

	static int iCol;
	static byte bPixel;

	// Check that there are no dialog boxes on filelist screen
	for ( iCol = 400; iCol <= 410; iCol++ ) {
		TAP_Osd_GetPixel( 0, iCol, 223, &bPixel );
		//  Check if grey bar exists
		if ( bPixel != 7 ) return TRUE;
	}
	return FALSE;
}

// Returns true if there is a dialog box open over the file list.
// There are several ways this can be true, such as having the "Delete File Yes/No" displayed,
// or having the Options menu up (blue button), or the Toppy "keyboard" open.
bool isDialogOverFileList(void) {
	// works by checking if the grey lines separating each file are continuous across
	// the centre of the screen. If they're not, then there is a dialog showing on top of them.
	
	word *rowA, *rowB, *rowC, *rowD;
	word colGrey = 0;
    dword x;
	TYPE_OsdBaseInfo osdBaseInfo;

    // set row addresses for the grey pixel rows that separate file slots near the
    // centre of the file list screen, according to Hardware type
    switch (hardwareType) {
	    case HW_5000PVRt:
	    	// get starting addresses
    		TAP_Osd_GetBaseInfo( &osdBaseInfo );
    		// 50805 = RGB(17,19,21) which is the blue-grey used on 5000PVRt as horizontal lines in file list
	    	colGrey = 50805;
	    	rowA = osdBaseInfo.oAddr + 720*196; //row 197
    		rowB = osdBaseInfo.eAddr + 720*198; //row 198
    		rowC = osdBaseInfo.oAddr + 720*250; //row 251
    		rowD = osdBaseInfo.eAddr + 720*252; //row 252
    		break;
	    case HW_MP: // Masterpiece - use different method
	    	return isDialogOverFileListMP();
    		break;
    	case HW_Procaster: // Procaster, as provided by Esa (uses different colour)
    		// get starting addresses
    		TAP_Osd_GetBaseInfo( &osdBaseInfo );
    		colGrey = RGB(24,24,24);
    		rowA = osdBaseInfo.oAddr + 720*198; //row 199
    		rowB = osdBaseInfo.eAddr + 720*200; //row 200
    		rowC = osdBaseInfo.oAddr + 720*252; //row 253
    		rowD = osdBaseInfo.eAddr + 720*254; //row 254
    		break;
    	case HW_5800: // To be implemented
    		//??
			return FALSE;
    		break;
    	case HW_None: // skip the check
    		return FALSE;
    	default: // shouldn't reach this code, provide pass-through just in case
    		hardwareType = HW_None;
    		return FALSE;
    		break;
	}
    // if any of these grey lines are broken, then there is a dialog over the top of them.
	for( x=180; x<550; x+=4 ) {
		if ( *(rowA+x) != colGrey ) return TRUE;
		if ( *(rowB+x) != colGrey ) return TRUE;
		if ( *(rowC+x) != colGrey ) return TRUE;
		if ( *(rowD+x) != colGrey ) return TRUE;
	}
	return FALSE;
}

