/*
*
*  Copyright (c) 2005 Michael Stock. All Rights Reserved.
*
*  This file is part of Stirf.
*
*  Stirf is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  Stirf is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this software; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// converts illegal characters (illegal in the windows filename sense) into legal ones
void convertIllegalChars (char* str) {
	
	char* ptr = str;
	
	while (*ptr != '\0') {
		switch (*ptr) {
			case 34: // double quote
				*ptr = 39; // single quote
				break;
			case 42: // asterisk
				*ptr = '#';
				break;
			case 47: // forward slash
				*ptr = '-';
				break;
			case 58: // CR or colon?
				*ptr = ';';
				break;
			case 60: // <
			case 62: // >
				*ptr = '-';
				break;
			case 63: // question mark
				*ptr = '!';
				break;
			case 92: // back slash
				*ptr = '-';
				break;
			case 124: // pipe
				*ptr = '!';
				break;
			default:
				break;
		}
		ptr++;
	}
	
}

// Kills binary data/ control characters from string
void KillCtrChar (unsigned char * SendText)
{
    int count, count2;
    count = 0;
    count2 = 0;

    while (SendText[count] != 0)
    {
         if  ((SendText[count] > 0x1F && SendText[count] < 0x7F) || (SendText[count] > 0x9F) || (SendText[count] == 0x8a))
            {
	            if (SendText[count] == 0x8a)
                	SendText[count2++] =  0x20;
                else
                	SendText[count2++] = SendText[count];
            }
        count++;
    }
    SendText[count2] = 0;
}
// prints the suggested filename into buff, for file "currItem" and menu item "posn".
// currItem used as index into subTitleRecords/fileRecords, posn as index into .ptrs
// Note posn can be +ve or -ve, with absolute value of up to (numPtrs +1) - it handles the "looping".
// buff must already have allocated to it enough space for TS_FILE_NAME_SIZE characters.
// If appendDotRec == TRUE then ".rec" is appended to the file name.
// If convertIllegals == TRUE then characters that are illegal in windows filenames are converted to legals.
//
// Added at v0.02: restrict filename length to 57 characters (before ".rec\0") to get around API bug
// Added at v0.04: only do this truncation when appendDotRec is true, so users can (during display only times) see the whole lot (to perhaps trim with)
//
void makeSuggestedFileName(char* buff, const int fileIndex, int posn, const bool appendDotRec, const bool convertIllegals, const bool trimWords) {

	byte numP;
	long len = 0; // we are going to use this for pointer arithmetic
	char* tempPtr = NULL;
	char* endPtr = NULL;
	char copyBuff[TS_FILE_NAME_SIZE];
	int trimmedLen = 0;
	int wordCounter = 0;

	memset(copyBuff, 0, sizeof copyBuff);
	buff[0] = '\0';
	numP = subTitleRecords[fileIndex].numPtrs;
	
	if ( (posn == 0) || (numP == 0) ) {
		switch (language) {
			case Lang_Finnish:
				strcpy(buff,"(Ei muutoksia ohjelman tai jakson nimeen)");
				break;
			case Lang_English:
			default:
				strcpy(buff,"(No change to filename or subtitle)");
				break;
		}
		return;
	}
	
	// capture the "off the end of the list" looping around behaviour
	if ( posn == (-1 - numP) ) {
		posn = 2*numP;
	} else if ( posn == (2*numP + 1) ) {
		posn = -numP;
	}
	// < 0 means prepend the title (and " - ") to the subtitle
	if (posn < 0) {
		len = (subTitleRecords[fileIndex].ptrs[((-posn)-1)]) - (titleRecords[fileIndex].ptrs[((-posn)-1)]);
		if ( (len > 0) && (len < (TS_FILE_NAME_SIZE - 6)) ) {
			strncpy(buff, titleRecords[fileIndex].ptrs[((-posn)-1)], len );
			buff[len] = '\0';
			// handle Procaster's extra 0x05 byte
			if (buff[len-1] == 5) {
				buff[len-1] = '\0';
			}
			strncat(buff, " - ", TS_FILE_NAME_SIZE - strlen(buff) - 5);
		}
		strncat(buff, subTitleRecords[fileIndex].ptrs[(-posn)-1], TS_FILE_NAME_SIZE - strlen(buff) - 5);
	} else if (posn <= numP) {
		// then we want the subtitle only as the suggested filename
		strncat(buff, subTitleRecords[fileIndex].ptrs[posn-1], TS_FILE_NAME_SIZE - strlen(buff) - 5);
	} else { //posn is between numP+1 and 2*numP (inclusive)
		// we want the trimmed filename plus the subtitle as the suggested filename
		trimmedLen = trimmedFileNameLen[fileIndex];
		strncpy(buff,fileNames[fileIndex],trimmedLen);
		buff[trimmedLen] = '\0';
		strncat(buff, " - ", TS_FILE_NAME_SIZE - strlen(buff) - 5);
		strncat(buff, subTitleRecords[fileIndex].ptrs[posn-numP-1], TS_FILE_NAME_SIZE - strlen(buff) - 5);
	}
	// check if there is genre info in the subtitle (i.e. a '[' character in there)
	tempPtr = strrchr(buff, '[');
	if ( (tempPtr != NULL) && (tempPtr != buff) ) {
		*tempPtr = '\0';
		// now trim any trailing spaces (and, in this case, hyphens - in case the subtitle was just genre info)
		len = strlen(buff);
		while ( (len > 2) && ((buff[len-1] == '-') || (buff[len-1] == ' ')) ) {
			buff[len-1] = '\0';
			len--;
		}
	}
	if (convertIllegals) {
		convertIllegalChars(buff);
		KillCtrChar(buff);
	}
	
	// Handle trimming of words off suggested file name
	if ( trimWords && ( (numWordsToTrimEnd[fileIndex]+numWordsToTrimStart[fileIndex]) > 0) ) {
		// Start with trimming the end (if necessary)
		endPtr = strchr(buff, '\0');
		while ( (endPtr > (buff+1)) && (wordCounter < numWordsToTrimEnd[fileIndex]) ) {
			endPtr--;
			if (*endPtr == ' ') {
				wordCounter ++;
			}
		}
		// check the resulting pointer location
		if ( (endPtr - buff) < 3 ) {
			// Then we didn't find enough words to trim, go back to trimming none
			numWordsToTrimEnd[fileIndex] = 0;
			numWordsToTrimStart[fileIndex] = 0;
		} else { // Finding spot to trim off end was successful
			// Now start trimming words off the front as necessary
			if (numWordsToTrimStart[fileIndex] > 0) {
				wordCounter = 0;
				tempPtr = buff;
				while ( (tempPtr < (endPtr-2)) && (wordCounter < numWordsToTrimStart[fileIndex]) ) {
					if (*tempPtr == ' ') {
						wordCounter ++;
					}
					tempPtr++;
				}
				// Now tempPtr is either at the first letter of the first word, or it's gone over
				if (tempPtr >= (endPtr-2)) {
					// Then we didn't find enough words to trim, go back to trimming none
					numWordsToTrimEnd[fileIndex] = 0;
					numWordsToTrimStart[fileIndex] = 0;
				} else {
					// Do the front and back trimming
					*endPtr = '\0'; // trim end
					// Copy to temporary area and back into buff
					strcpy(copyBuff,tempPtr);
					buff[0] = '\0';
					strcpy(buff,copyBuff);
				}
			} else {
				// Just trim the word(s) off the end
				*endPtr = '\0';
			}
		}
		
		// Clean up by again trimming any trailing spaces resulting from the filename trimming etc
		len = strlen(buff);
		while ( (len > 2) && (buff[len-1] == ' ') ) {
			buff[len-1] = '\0';
			len--;
		}
	}
	
	if (appendDotRec) {
		// trim the file name according to unpublished API problem: toppy can't handle
		// file names more than 64 bytes including ".rec\0". Restrict to 57 before ".rec\0"
		buff[57] = '\0';
		// now trim any trailing spaces resulting from the filename length truncation
		len = strlen(buff);
		while ( (len > 2) && (buff[len-1] == ' ') ) {
			buff[len-1] = '\0';
			len--;
		}
		strcat(buff, ".rec");
	}
}

// checks value at posn in array buffer, to see if it points at a capital letter with
// a lower case letter before it.
// DOES NO ERROR CHECKING BEFORE REFERENCING ARRAYS!! (done outside this function)
bool isLowerUpper(const byte* buff, const int posn) {
	if ( (buff[posn] >= 65) && (buff[posn] <= 90) && ( ((buff[posn-1] >= 224) && (buff[posn-1] <= 255)) || ((buff[posn-1] >= 97) && (buff[posn-1] <= 122)) ) ) {
		return TRUE;
	}
	return FALSE;
}

// Checks the string segment to see if it is a PBK datestamp (with no leading or trailing space).
// Returns 1 if it finds a datestamp, 0 if not.
// Note: Will not detect ##-##-##, but will detect the first part of that (i.e. ##-##), this is
//       because the last bit (-##) will already have been removed as a DirLister stamp by getTrimmedFileLength
bool isPBKdateStamp( const char* segment ) {
	
	int len = strlen(segment);
	switch (len) {
		case 3:
			if ( strcmp(segment,"Mon") == 0) return TRUE;
			if ( strcmp(segment,"Tue") == 0) return TRUE;
			if ( strcmp(segment,"Wed") == 0) return TRUE;
			if ( strcmp(segment,"Thu") == 0) return TRUE;
			if ( strcmp(segment,"Fri") == 0) return TRUE;
			if ( strcmp(segment,"Sat") == 0) return TRUE;
			if ( strcmp(segment,"Sun") == 0) return TRUE;
			return FALSE;
			break;
		case 5: // looking for "##/##" or "##-##"
			if ( (segment[2] == '/') || (segment[2] == '-') ) {
				// all the other characters should be digits. They will be double-digit numbers, and the
				// first digit must be at most 3 which is ASCII char 51 (no more than 39 days in a month,
				// or months in a year, and i don't think anyone will be using this in 2040)
				if ( (segment[0] >= 48) && (segment[0] <= 51) && (segment[1] >= 48) && (segment[1] <= 57) && (segment[3] >= 48) && (segment[3] <= 51) && (segment[4] >= 48) && (segment[4] <= 57) ) {
					return TRUE;
				}
			}
			return FALSE;
			break;
		case 6:
			// similar to before, this time we're looking for "######": three double digit numbers all 39 or less (i.e. first digit 3)
			if ( (segment[0] >= 48) && (segment[0] <= 51) && (segment[1] >= 48) && (segment[1] <= 57) && (segment[2] >= 48) && (segment[2] <= 51) && (segment[3] >= 48) && (segment[3] <= 57) && (segment[4] >= 48) && (segment[4] <= 51) && (segment[5] >= 48) && (segment[5] <= 57) ) {
				return TRUE;
			}
			return FALSE;
			break;
		case 8:
			if ( (segment[2] == '/') && (segment[5] == '/') ) {
				// similar to before, this time we're looking for "##/##/##": three double digit numbers (separated by /) all 39 or less (i.e. first digit 3)
				if ( (segment[0] >= 48) && (segment[0] <= 51) && (segment[1] >= 48) && (segment[1] <= 57) && (segment[3] >= 48) && (segment[3] <= 51) && (segment[4] >= 48) && (segment[4] <= 57) && (segment[6] >= 48) && (segment[6] <= 51) && (segment[7] >= 48) && (segment[7] <= 57) ) {
					return TRUE;
				}
			}
			return FALSE;
			break;
		default:
			return FALSE;
	}
}

//////////////// getTrimmedFileNameLength /////////////////////
//
// Determines the length of the "real" part of the filename.
// The real part does NOT include:
//      ".rec" at the end
//      "-#" at the end (from TimerExtend)
//      "-1" through "-1231" at the end (from DirLister, file number or datestamp MMDD)
//      " (1)" through " (9)" at the end (from DirLister when >1 file has same date)
//      " 0101" through " 1231" at the end (from PBK, datestamp MMDD)
//      " ##/##/##" at the end (from PBK, where the #'s are digits in the datestamp)
//      " ##-##-##" at the end (from PBK, where the #'s are digits in the datestamp)
//      " ######" at the end (from PBK, where the #'s are digits in the datestamp)
//      " ##/##" at the end (from PBK, where the #'s are digits in the datestamp)
//      " Mon" through " Sun" at the end (from PBK, daystamp)
//      "_##-##-##" at the end (from Jag's EPG - datestamp)
//
// Note that the contents of fName are not modified in this function. fName must not be
// more than TS_FILE_NAME_SIZE long, and must have ".rec" at the end.
int getTrimmedFileNameLength(const char* fName) {
	
	char buff[TS_FILE_NAME_SIZE];
	int span = 0;
	int length = 0;
	int number = 0;
	char* tempPtr = NULL;
	char* tempPtr2 = NULL;
	
	memset(buff, 0, sizeof buff);
	strcpy(buff,fName);
	// remove ".rec"
	length = strlen(fName) - 4;
	if (length < 0) return length+4;
	buff[length] = '\0';
	// trim trailing spaces (if any)
	while ( (buff[length-1] == ' ') && (length > 1) ) {
		buff[length-1] = '\0';
		length--;
	}
	// remove Dirlister tags of " (#)" where # is any number
	if ( (length > 4) && (buff[length-1] == ')') ) {
		if ( (buff[length-4] == ' ') && (buff[length-3] == '(') && (buff[length-2] > 48) && (buff[length-2] < 58) ) {
			//then we found a " (#)" at the end
			buff[length-4] = '\0';
			length -= 4;
			// trim trailing spaces (if any)
			while ( (buff[length-1] == ' ') && (length > 1) ) {
				buff[length-1] = '\0';
				length--;
			}
		}
	}
	// remove "-#" from the end, if present from TimerExtend
	if ( (length > 3) && (buff[length-2] == '-') && (buff[length-1] == '#') ) {
		buff[length-2] = '\0';
		length -= 2;
		// trim trailing spaces (if any)
		while ( (buff[length-1] == ' ') && (length > 1) ) {
			buff[length-1] = '\0';
			length--;
		}
	}
	// Find occurence of "-1" through "-1231" at the end. These are from DirLister, either just
	// numbering the files or putting a datestamp (hence don't remove numbers > 1231 as this is Dec 31st).
	// Start by finding the last hyphen in the filename and check after for digits
	tempPtr = strrchr(buff, '-');
	if ( (tempPtr != NULL) && (length > 3) ) {
		// We found a hyphen. Check if it is close to the end
		if ( (buff + length - tempPtr) <= 5 ) {
			// Check if the remaining bit of the string is all numbers
			span = strspn(tempPtr + 1,"0123456789");
			if ( (span == (buff + length - tempPtr - 1)) && (span <= 4) && (span > 0) ) {
				// then it's all numbers to the end, and no more than 4 of them
				number = GetInteger(tempPtr + 1);
				// number should be less than 1231 (Dec 31st) if it's a stamp or a file number
				if (number <= 1231) {
					// trim the number and the hyphen off
					*tempPtr = '\0';
					length = strlen(buff);
					// trim trailing spaces (if any)
					while ( (buff[length-1] == ' ') && (length > 1) ) {
						buff[length-1] = '\0';
						length--;
					}
				}
			}
		}
	}
	// Find occurence of PBK/JagEPG datestamps at the end (after a space or an underscore "_").
	tempPtr = strrchr(buff, ' ');
	tempPtr2 = strrchr(buff, '_');
	tempPtr = max(tempPtr,tempPtr2);
	if ( (tempPtr != NULL) && (length > 4) ) {
		// We found a space. Check if it is close to the end
		if ( (buff + length - tempPtr) <= 9 ) {
			// Check if the remaining bit of the string is a PBK datestamp
			if ( isPBKdateStamp(tempPtr + 1) ) {
				*tempPtr = '\0';
				length = strlen(buff);
				// trim trailing spaces (if any)
				while ( (buff[length-1] == ' ') && (length > 1) ) {
					buff[length-1] = '\0';
					length--;
				}
			}
		}
	}
	return length;
}


// Returns TRUE if the strings are equal as far as the shortest one's length, i.e. if one is a starting sub-string of the other.
// Note: returns TRUE if they are exactly the same.
bool isStartingSubStr (const char* str1, const char* str2) {
	
	int i = 0;
	
	while ( (str1[i] != '\0') && (str2[i] != '\0') ) {
		if ( str1[i] != str2[i] ) return FALSE;
		i++;
	}
	
	return TRUE;
	
}

// Looks at the string and tries to match the start of it with one of the titles
// in the global variable specialShowTitles which is ';' delimited. Returns the length of the show title
// that matched, or 0 if none.
int getSpecialTitleLength(char* titleAndSubtitle) {
	
	char* titleStart = specialShowTitles;
	char* titleFinish = strchr(specialShowTitles,';');
	int titleLength = 0;
	
	while (titleFinish != NULL) {
		titleLength = titleFinish - titleStart;
		if ( (titleLength > 0) && (strncmp(titleStart,titleAndSubtitle,titleLength) == 0) ) {
			return titleLength;
		}
		// move to next title in specialShowTitles
		titleStart = titleFinish + 1;
		titleFinish = strchr(titleStart,';'); // returns NULL if no ';' found
	}
	// if we didn't find any:	
	return 0;	
}

// Modified from Peter Gillespie's IsFileRec routine.
bool	IsFileEPG( char *fileName )
{
	//modified by mike
	
	char sBuff[4];
	int len;
	
	len = strlen(fileName);
	if (len > 4) {
		strcpy( sBuff, fileName + len - 4);
		strlwr( sBuff);
		if (strcmp(sBuff, ".epg") == 0)
			return TRUE;
	}
	
	return FALSE;
	
}
